"""Ceremony (medal ranking) command handler."""

from __future__ import annotations

import argparse
import re
import sys

from ..api import BiathlonError, get_current_season_id, get_events, get_race_results, get_races, get_seasons
from ..constants import RELAY_DISCIPLINE
from ..formatting import is_pretty_output, rank_style, render_table
from ..utils import extract_results


def accumulate_medal_counts(race_ids: list[str], by_athlete: bool, gender_filter: str = "") -> tuple[list[dict], int]:
    """Aggregate podium counts across races."""
    counts: dict[str, dict] = {}
    total_used = 0

    for race_id in race_ids:
        try:
            payload = get_race_results(race_id)
        except BiathlonError:
            continue
        comp = payload.get("Competition") or {}
        discipline = str(comp.get("DisciplineId") or "").upper()
        if discipline == RELAY_DISCIPLINE:
            continue
        results = extract_results(payload)
        if not results:
            continue
        if gender_filter:
            cat = str(comp.get("catId") or comp.get("CatId") or "").upper()
            if gender_filter == "women" and cat not in {"SW", ""}:
                continue
            if gender_filter == "men" and cat not in {"SM", ""}:
                continue
        total_used += 1
        top = results[:6]
        for idx, res in enumerate(top):
            nat = res.get("Nat") or ""
            if by_athlete:
                key = res.get("Name") or res.get("ShortName") or ""
                label = key
                counts.setdefault(key, {
                    "label": label, "country": nat,
                    "first": 0, "second": 0, "third": 0, "fourth": 0, "fifth": 0, "sixth": 0,
                })
            else:
                key = nat
                label = nat
                counts.setdefault(key, {
                    "label": label, "country": "",
                    "first": 0, "second": 0, "third": 0, "fourth": 0, "fifth": 0, "sixth": 0,
                })
            slot = ["first", "second", "third", "fourth", "fifth", "sixth"][idx]
            counts[key][slot] += 1

    rows = list(counts.values())
    rows.sort(key=lambda row: (
        -row["first"], -row["second"], -row["third"], -row["fourth"], -row["fifth"], -row["sixth"], row["label"]
    ))
    return rows, total_used


def handle_ceremony(args: argparse.Namespace) -> int:
    """Show medal/placing ranking by country or athlete."""
    scope_count = sum(1 for v in [args.race, args.event] if v)
    if scope_count > 1:
        print("error: use only one of --race or --event", file=sys.stderr)
        return 1

    by_athlete = args.athlete
    gender_filter = "men" if args.men else "women" if getattr(args, "women", False) else ""
    season_id = args.season or get_current_season_id()

    def normalize_country(value: str) -> str:
        v = (value or "").strip()
        if not v:
            return ""
        return v[:3].upper() if len(v) > 3 else v.upper()

    country_filter = normalize_country(args.country or "")
    race_ids: list[str] = []
    scope_label = ""

    if args.race:
        include_race = True
        if country_filter:
            try:
                payload = get_race_results(args.race)
            except BiathlonError:
                payload = {}
            comp = payload.get("Competition") or {}
            sport_evt = payload.get("SportEvt") or {}
            race_nat = normalize_country(
                comp.get("Nat") or comp.get("Nation") or sport_evt.get("Nat") or sport_evt.get("Nation") or ""
            )
            include_race = bool(race_nat) and race_nat == country_filter
        if include_race:
            race_ids = [args.race]
        scope_label = args.race
    elif args.event:
        event_ok = True
        if country_filter:
            inferred_season = season_id
            if not args.season:
                match = re.search(r"BT(\d{4})", args.event)
                if match:
                    inferred_season = match.group(1)
            try:
                ev_list = get_events(inferred_season, level=1)
            except BiathlonError:
                ev_list = []
            ev_match = next((ev for ev in ev_list if ev.get("EventId") == args.event), None)
            ev_nat = normalize_country(
                ev_match.get("Nat") or ev_match.get("Nation") or ev_match.get("CountryId") or ev_match.get("Country") or ""
            ) if ev_match else ""
            if ev_nat and ev_nat != country_filter:
                event_ok = False
        if event_ok:
            for race in get_races(args.event):
                race_id = race.get("RaceId") or race.get("Id")
                if race_id:
                    race_ids.append(race_id)
        scope_label = args.event
    else:
        season_ids: list[str]
        search_filter = (args.search or "").strip().lower()
        # Search across all seasons when filtering by country or search term
        if country_filter or search_filter:
            season_ids = [str(season.get("SeasonId")) for season in get_seasons()]
            if search_filter and country_filter:
                scope_label = f"World Cup '{args.search}' in {args.country}"
            elif search_filter:
                scope_label = f"World Cup '{args.search}'"
            else:
                scope_label = f"World Cup in {args.country}"
        else:
            season_ids = [season_id]
            scope_label = f"World Cup season {season_id}"
        events: list[dict] = []
        for sid in season_ids:
            events.extend(get_events(sid, level=1))
        for event in events:
            ev_nat = normalize_country(
                event.get("Nat") or event.get("Nation") or event.get("CountryId") or event.get("Country") or ""
            )
            if country_filter and ev_nat != country_filter:
                continue
            # Filter by search term (event name)
            if search_filter:
                ev_name = (
                    (event.get("ShortDescription") or "") + " " +
                    (event.get("Organizer") or "") + " " +
                    (event.get("Description") or "")
                ).lower()
                if search_filter not in ev_name:
                    continue
            event_id = event.get("EventId")
            if not event_id:
                continue
            for race in get_races(event_id):
                race_id = race.get("RaceId") or race.get("Id")
                if race_id:
                    race_ids.append(race_id)

    rows, used_races = accumulate_medal_counts(race_ids, by_athlete, gender_filter)
    if used_races == 0:
        print("no completed races found for ceremony ranking", file=sys.stderr)
        return 1
    if not rows:
        print("no medal data found for the requested scope", file=sys.stderr)
        return 1

    headers = ["Rank", "Country" if not by_athlete else "Name"]
    if by_athlete:
        headers.append("Nat")
    headers += ["Gold", "Silver", "Bronze", "Podium", "Fourth", "Fifth", "Sixth", "Flowers", "Total"]

    render_rows = []
    for idx, row in enumerate(rows, start=1):
        base = [idx, row["label"]]
        if by_athlete:
            base.append(row["country"])
        podium = row["first"] + row["second"] + row["third"]
        flowers = row["fourth"] + row["fifth"] + row["sixth"]
        base.extend([
            row["first"], row["second"], row["third"], podium,
            row["fourth"], row["fifth"], row["sixth"], flowers,
            podium + flowers,
        ])
        render_rows.append(base)

    pretty = is_pretty_output(args)
    row_styles = [rank_style(idx + 1) for idx in range(len(render_rows))] if pretty else None

    gender_label = "men" if gender_filter == "men" else "women" if gender_filter == "women" else "men+women"
    group_label = "by athlete" if by_athlete else "by country"

    print()
    print(f"# Medal ranking — {scope_label} — {gender_label}, {group_label} ({used_races} races)")
    render_table(headers, render_rows, pretty=pretty, row_styles=row_styles)
    print()
    return 0



"""Seasons command handler."""

from __future__ import annotations

import argparse

from ..api import fetch_json
from ..formatting import is_pretty_output, render_table


def compute_season_styles(seasons: list[dict]) -> list[str]:
    """Return style names for each season row: dim for past, highlight for current."""
    styles: list[str] = []
    current_sort_order: int | None = None
    for season in seasons:
        if season.get("IsCurrent"):
            current_sort_order = season.get("SortOrder", 0)
            break

    for season in seasons:
        sort_order = season.get("SortOrder", 0)
        if season.get("IsCurrent"):
            styles.append("highlight")
        elif current_sort_order is not None and sort_order < current_sort_order:
            styles.append("dim")
        else:
            styles.append("")
    return styles


def handle_seasons(args: argparse.Namespace) -> int:
    """List all seasons available on the API."""
    seasons = fetch_json("Seasons")
    sorted_seasons = sorted(
        seasons,
        key=lambda season: season.get("SortOrder", 0),
        reverse=True,
    )

    # Apply display limit
    limit_n = getattr(args, "limit", 25) or 0
    if limit_n > 0:
        sorted_seasons = sorted_seasons[:limit_n]

    pretty = is_pretty_output(args)
    rows: list[list[str]] = []
    for season in sorted_seasons:
        season_id = season.get("SeasonId", "")
        desc = season.get("Description", "")
        if season.get("IsCurrent"):
            desc = f"{desc} *"
        rows.append([season_id, desc])

    row_styles = compute_season_styles(sorted_seasons) if pretty else None
    render_table(["Season", "Description"], rows, pretty=pretty, row_styles=row_styles)
    return 0



"""Output formatting utilities for the Biathlon CLI."""

from __future__ import annotations

import os
import re
import sys
import unicodedata
from collections.abc import Callable


class Color:
    """ANSI color codes for terminal output."""

    RESET = "\033[0m"
    DIM = "\033[2m"
    BOLD = "\033[1m"
    OCEAN_BLUE = (79, 193, 255)
    SECTION_TITLE = (255, 170, 0)
    ACCURACY_BANDS = [
        (0.0, 60.0, (176, 0, 32), (176, 0, 32)),       # #B00020
        (60.0, 75.0, (176, 0, 32), (230, 81, 0)),       # #E65100
        (75.0, 85.0, (230, 81, 0), (255, 214, 0)),      # #FFD600
        (85.0, 92.0, (255, 214, 0), (174, 234, 0)),     # #AEEA00
        (92.0, 97.0, (174, 234, 0), (0, 200, 83)),      # #00C853
        (97.0, 100.0, (0, 200, 83), (0, 230, 118)),     # #00E676
    ]
    COLOR_PREFIX = "\033[38;2;"
    COLOR_SUFFIX = "m"

    GOLD = (255, 215, 0)
    LIGHT_GOLD = (218, 165, 32)  # Goldenrod - distinct from bold gold
    SILVER = (192, 192, 192)
    BRONZE = (205, 127, 50)
    FLOWERS = (255, 182, 108)
    OTHER = (215, 215, 215)
    GREEN = (0, 200, 0)
    RED = (220, 60, 60)

    @classmethod
    def enabled(cls) -> bool:
        """Check if colors should be enabled."""
        if os.environ.get("NO_COLOR"):
            return False
        if not sys.stdout.isatty():
            return False
        return True

    @classmethod
    def dim(cls, text: str) -> str:
        """Apply dim style (for past events)."""
        if not cls.enabled():
            return text
        return f"{cls.DIM}{text}{cls.RESET}"

    @classmethod
    def highlight(cls, text: str) -> str:
        """Apply highlight style (for current/next event)."""
        if not cls.enabled():
            return text
        return cls.rgb(text, cls.OCEAN_BLUE, bold=True)

    @classmethod
    def section_title(cls, text: str) -> str:
        """Apply section title style."""
        if not cls.enabled():
            return text
        return cls.rgb(text, cls.SECTION_TITLE, bold=True)

    @classmethod
    def highlight_soft(cls, text: str) -> str:
        """Apply a softer highlight style for related columns."""
        return cls.rgb(text, (0, 170, 0), bold=False)

    @classmethod
    def rgb(cls, text: str, color: tuple[int, int, int], bold: bool = False) -> str:
        """Apply a 24-bit color (optionally bold)."""
        if not cls.enabled():
            return text
        r, g, b = color
        prefix = f"{cls.BOLD}" if bold else ""
        return f"{prefix}{cls.COLOR_PREFIX}{r};{g};{b}{cls.COLOR_SUFFIX}{text}{cls.RESET}"

    @classmethod
    def silver(cls, text: str) -> str:
        """Apply silver style (2nd place)."""
        return cls.rgb(text, cls.SILVER, bold=True)

    @classmethod
    def bronze(cls, text: str) -> str:
        """Apply bronze style (3rd place)."""
        return cls.rgb(text, cls.BRONZE, bold=True)

    @classmethod
    def gold(cls, text: str) -> str:
        """Apply gold style (1st place)."""
        return cls.rgb(text, cls.GOLD, bold=True)

    @classmethod
    def flowers(cls, text: str) -> str:
        """Apply flowers ceremony style (4th/5th/6th place)."""
        return cls.rgb(text, cls.FLOWERS, bold=False)

    @classmethod
    def other(cls, text: str) -> str:
        """Apply style for non-top athletes."""
        return cls.dim(text)

    @classmethod
    def green(cls, text: str, intensity: float = 1.0) -> str:
        """Apply green color with intensity scale (0.0 to 1.0)."""
        base_g = 100
        max_g = 220
        g = int(base_g + (max_g - base_g) * min(1.0, max(0.0, intensity)))
        return cls.rgb(text, (0, g, 0), bold=intensity > 0.5)

    @classmethod
    def red(cls, text: str, intensity: float = 1.0) -> str:
        """Apply red color with intensity scale (0.0 to 1.0)."""
        base_r = 150
        max_r = 240
        r = int(base_r + (max_r - base_r) * min(1.0, max(0.0, intensity)))
        return cls.rgb(text, (r, 50, 50), bold=intensity > 0.5)

    @classmethod
    def accuracy(cls, text: str, pct: float) -> str:
        """Apply color based on accuracy percentage (0.0 to 1.0)."""
        if not cls.enabled():
            return text
        percent = max(0.0, min(100.0, pct * 100.0))
        for low, high, low_color, high_color in cls.ACCURACY_BANDS:
            if percent <= high:
                if high == low:
                    return cls.rgb(text, high_color)
                t = (percent - low) / (high - low)
                color = cls._interp_color(low_color, high_color, t)
                return cls.rgb(text, color)
        return cls.rgb(text, cls.ACCURACY_BANDS[-1][3])

    @classmethod
    def _apply_bands(cls, text: str, value: float, bands: list) -> str:
        """Apply color from a band list based on raw value."""
        if not cls.enabled():
            return text
        for low, high, low_color, high_color in bands:
            if value <= high:
                if high == low:
                    return cls.rgb(text, high_color)
                t = (value - low) / (high - low)
                color = cls._interp_color(low_color, high_color, t)
                return cls.rgb(text, color)
        return cls.rgb(text, bands[-1][3])

    @classmethod
    def clean_race_pct(cls, text: str, pct: float) -> str:
        """Apply color based on clean race percentage (0.0 to 1.0)."""
        return cls._apply_bands(text, max(0.0, min(100.0, pct * 100.0)), cls.CLEAN_RACE_PCT_BANDS)

    @classmethod
    def clean_stage_pct(cls, text: str, pct: float) -> str:
        """Apply color based on clean stage percentage (0.0 to 1.0)."""
        return cls._apply_bands(text, max(0.0, min(100.0, pct * 100.0)), cls.CLEAN_STAGE_PCT_BANDS)

    @classmethod
    def shoot_time(cls, text: str, seconds: float) -> str:
        """Apply color based on avg stage shoot time in seconds (lower is better)."""
        return cls._apply_bands(text, seconds, cls.SHOOT_TIME_BANDS)

    @classmethod
    def range_time(cls, text: str, seconds: float) -> str:
        """Apply color based on avg stage range time in seconds (lower is better)."""
        return cls._apply_bands(text, seconds, cls.RANGE_TIME_BANDS)

    # Clean race % bands – tuned for skewed-low distribution (median ~0-8%)
    CLEAN_RACE_PCT_BANDS = [
        (0.0, 5.0, (176, 0, 32), (176, 0, 32)),
        (5.0, 15.0, (176, 0, 32), (230, 81, 0)),
        (15.0, 30.0, (230, 81, 0), (255, 214, 0)),
        (30.0, 45.0, (255, 214, 0), (174, 234, 0)),
        (45.0, 65.0, (174, 234, 0), (0, 200, 83)),
        (65.0, 100.0, (0, 200, 83), (0, 230, 118)),
    ]

    # Clean stage % bands – median ~37%, spread 0-100%
    CLEAN_STAGE_PCT_BANDS = [
        (0.0, 20.0, (176, 0, 32), (176, 0, 32)),
        (20.0, 35.0, (176, 0, 32), (230, 81, 0)),
        (35.0, 50.0, (230, 81, 0), (255, 214, 0)),
        (50.0, 60.0, (255, 214, 0), (174, 234, 0)),
        (60.0, 75.0, (174, 234, 0), (0, 200, 83)),
        (75.0, 100.0, (0, 200, 83), (0, 230, 118)),
    ]

    # Avg stage shoot time bands (seconds, lower = better) – typical 24-37s
    SHOOT_TIME_BANDS = [
        (0.0, 24.0, (0, 230, 118), (0, 230, 118)),
        (24.0, 27.0, (0, 200, 83), (174, 234, 0)),
        (27.0, 30.0, (174, 234, 0), (255, 214, 0)),
        (30.0, 33.0, (255, 214, 0), (230, 81, 0)),
        (33.0, 37.0, (230, 81, 0), (176, 0, 32)),
        (37.0, 999.0, (176, 0, 32), (176, 0, 32)),
    ]

    # Avg stage range time bands (seconds, lower = better) – typical 44-62s
    RANGE_TIME_BANDS = [
        (0.0, 44.0, (0, 230, 118), (0, 230, 118)),
        (44.0, 48.0, (0, 200, 83), (174, 234, 0)),
        (48.0, 52.0, (174, 234, 0), (255, 214, 0)),
        (52.0, 56.0, (255, 214, 0), (230, 81, 0)),
        (56.0, 62.0, (230, 81, 0), (176, 0, 32)),
        (62.0, 999.0, (176, 0, 32), (176, 0, 32)),
    ]

    RELATIVE_BANDS = [
        (0.0, 25.0, (176, 0, 32), (230, 81, 0)),
        (25.0, 50.0, (230, 81, 0), (255, 214, 0)),
        (50.0, 75.0, (255, 214, 0), (174, 234, 0)),
        (75.0, 100.0, (174, 234, 0), (0, 230, 118)),
    ]

    @classmethod
    def relative(cls, text: str, t: float) -> str:
        """Apply color based on a relative 0.0-1.0 scale spread evenly across the range."""
        if not cls.enabled():
            return text
        percent = max(0.0, min(100.0, t * 100.0))
        for low, high, low_color, high_color in cls.RELATIVE_BANDS:
            if percent <= high:
                frac = (percent - low) / (high - low) if high > low else 1.0
                color = cls._interp_color(low_color, high_color, frac)
                return cls.rgb(text, color)
        return cls.rgb(text, cls.RELATIVE_BANDS[-1][3])

    @staticmethod
    def _interp_color(low: tuple[int, int, int], high: tuple[int, int, int], t: float) -> tuple[int, int, int]:
        t = max(0.0, min(1.0, t))
        return (
            int(low[0] + (high[0] - low[0]) * t),
            int(low[1] + (high[1] - low[1]) * t),
            int(low[2] + (high[2] - low[2]) * t),
        )


_ANSI_RE = re.compile(r"\x1b\[[0-9;]*m")
_ZERO_WIDTH_CODEPOINTS = {0xFE0E, 0xFE0F}


def _strip_ansi(text: str) -> str:
    return _ANSI_RE.sub("", text)


def _display_width(text: str) -> int:
    width = 0
    stripped = _strip_ansi(text)
    idx = 0
    while idx < len(stripped):
        ch = stripped[idx]
        code = ord(ch)
        if code in _ZERO_WIDTH_CODEPOINTS:
            idx += 1
            continue
        if unicodedata.combining(ch):
            idx += 1
            continue
        if unicodedata.category(ch) in {"Mn", "Cf"}:
            idx += 1
            continue
        if unicodedata.east_asian_width(ch) in {"W", "F"}:
            width += 2
        else:
            width += 1
        idx += 1
    return width


def _pad_cell(text: str, width: int, alignment: str = "left") -> str:
    pad_len = width - _display_width(text)
    if pad_len <= 0:
        return text
    if alignment == "right":
        return f"{' ' * pad_len}{text}"
    if alignment == "center":
        left = pad_len // 2
        right = pad_len - left
        return f"{' ' * left}{text}{' ' * right}"
    return f"{text}{' ' * pad_len}"


def render_table(
    headers: list[str],
    rows: list[list[str]],
    pretty: bool,
    row_styles: list[str] | None = None,
    cell_formatters: list[Callable] | None = None,
    alignments: list[str] | None = None,
    highlight_headers: list[int] | None = None,
    highlight_header_styles: dict[int, str] | None = None,
    show_headers: bool = True,
    column_separators: set[int] | None = None,
) -> None:
    """Render tabular data either aligned (pretty) or TSV.

    Args:
        headers: Column headers.
        rows: Data rows.
        pretty: If True, align columns; otherwise output TSV.
        row_styles: Optional list of style names per row ("dim", "highlight", or "").
        cell_formatters: Optional list of functions (one per column) to format cell values.
            Each function takes (value, row_index) and returns formatted string.
        alignments: Optional list of alignment directives ("left", "right", "center") per column.
        highlight_headers: Optional list of column indices to highlight in the header row.
        show_headers: If False, skip printing the header row (default True).
        column_separators: Optional set of column indices before which a vertical separator is drawn.
    """
    if not pretty:
        if show_headers:
            print("\t".join(headers))
        for row in rows:
            print("\t".join(str(cell) for cell in row))
        return

    def apply_row_style(text: str, style: str) -> str:
        if style == "dim":
            return Color.dim(text)
        if style == "highlight":
            return Color.highlight(text)
        if style == "gold":
            return Color.gold(text)
        if style == "silver":
            return Color.silver(text)
        if style == "bronze":
            return Color.bronze(text)
        if style == "flowers":
            return Color.flowers(text)
        if style == "red":
            return Color.red(text)
        if style == "other":
            return Color.other(text)
        return text

    formatted_rows: list[list[str]] = []
    for row_idx, row in enumerate(rows):
        style = ""
        if row_styles and row_idx < len(row_styles):
            style = row_styles[row_idx]

        formatted_row = []
        for col_idx, cell in enumerate(row):
            cell_str = str(cell)
            has_formatter = (
                cell_formatters
                and col_idx < len(cell_formatters)
                and cell_formatters[col_idx]
            )
            if has_formatter:
                cell_str = cell_formatters[col_idx](cell_str, row_idx)
            elif style:
                cell_str = apply_row_style(cell_str, style)
            formatted_row.append(cell_str)
        formatted_rows.append(formatted_row)

    widths = [
        max(
            _display_width(str(headers[idx])),
            max((_display_width(row[idx]) for row in formatted_rows), default=0),
        )
        for idx in range(len(headers))
    ]

    sep = column_separators or set()

    def _join(parts: list[str]) -> str:
        if not sep:
            return "  ".join(parts)
        pieces = []
        for i, part in enumerate(parts):
            if i > 0:
                pieces.append(" | " if i in sep else "  ")
            pieces.append(part)
        return "".join(pieces)

    def fmt_row(row_idx: int) -> str:
        parts = []
        for col_idx, cell_str in enumerate(formatted_rows[row_idx]):
            alignment = "left"
            if alignments and col_idx < len(alignments) and alignments[col_idx]:
                alignment = alignments[col_idx]
            parts.append(_pad_cell(cell_str, widths[col_idx], alignment))
        return _join(parts)

    def fmt_header(idx: int, h: str) -> str:
        alignment = "left"
        if alignments and idx < len(alignments) and alignments[idx]:
            alignment = alignments[idx]
        text = _pad_cell(str(h), widths[idx], alignment)
        if highlight_header_styles and idx in highlight_header_styles:
            style = highlight_header_styles[idx]
            if style == "highlight":
                return Color.highlight(text)
            if style == "highlight_soft":
                return Color.highlight_soft(text)
        if highlight_headers and idx in highlight_headers:
            return Color.highlight(text)
        # Bold headers by default
        return f"{Color.BOLD}{text}{Color.RESET}"

    if show_headers:
        header_parts = [fmt_header(i, h) for i, h in enumerate(headers)]
        print(_join(header_parts))
        if sep:
            dash_parts = ["-" * widths[i] for i in range(len(headers))]
            pieces = []
            for i, part in enumerate(dash_parts):
                if i > 0:
                    pieces.append("-+-" if i in sep else "--")
                pieces.append(part)
            print("".join(pieces))
    for idx in range(len(rows)):
        print(fmt_row(idx))


def format_seconds(seconds: float | None) -> str:
    """Render seconds as mm:ss.t or hh:mm:ss.t if needed."""
    if seconds is None:
        return "-"
    hours = int(seconds // 3600)
    remainder = seconds - hours * 3600
    minutes = int(remainder // 60)
    secs = remainder - minutes * 60
    if hours:
        return f"{hours:d}:{minutes:02d}:{secs:04.1f}"
    return f"{minutes:d}:{secs:04.1f}"


def format_pct(numerator: int, denominator: int) -> str:
    """Format a percentage with one decimal place."""
    if denominator == 0:
        return "-"
    return f"{100 * numerator / denominator:.1f}%"


def is_pretty_output(args) -> bool:
    """Return True if output should be pretty-printed (not TSV)."""
    return not getattr(args, "tsv", False)


def rank_style(rank: int | object) -> str:
    """Return style name for a given rank (1-6 get podium/flowers colors)."""
    try:
        r = int(rank)  # type: ignore[arg-type]
    except (TypeError, ValueError):
        return "other"
    return {
        1: "gold",
        2: "silver",
        3: "bronze",
        4: "flowers",
        5: "flowers",
        6: "flowers",
    }.get(r, "other")



"""Common utilities for the Biathlon CLI."""

from __future__ import annotations

import datetime
import re

from .api import BiathlonError, get_analytic_results, get_race_results
from .formatting import format_seconds


def parse_date(value: str | None) -> datetime.date | None:
    """Parse an ISO date string (with optional time) to a date object."""
    if not value:
        return None
    try:
        return datetime.date.fromisoformat(value.split("T", 1)[0])
    except ValueError:
        return None


def parse_start_datetime(value: str | None) -> datetime.datetime | None:
    """Parse an ISO datetime string (allowing Z suffix) into a timezone-aware datetime."""
    if not value:
        return None
    text = value.strip()
    if not text:
        return None
    # Replace Z with +00:00 for proper timezone parsing
    if text.endswith("Z"):
        text = text[:-1] + "+00:00"
    try:
        if "T" in text:
            dt = datetime.datetime.fromisoformat(text)
        else:
            dt = datetime.datetime.fromisoformat(f"{text}T00:00:00")
        # Ensure timezone-aware (assume UTC if naive)
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=datetime.timezone.utc)
        return dt
    except ValueError:
        return None


def parse_time_seconds(value: str | None) -> float | None:
    """Convert a time string to seconds (supports +diff, mm:ss.d, hh:mm:ss.d)."""
    if not value:
        return None
    text = value.strip()
    if text.startswith("+"):
        text = text[1:]
    parts = text.split(":")
    try:
        if len(parts) == 1:
            return float(parts[0])
        if len(parts) == 2:
            minutes = int(parts[0])
            seconds = float(parts[1])
            return minutes * 60 + seconds
        if len(parts) == 3:
            hours = int(parts[0])
            minutes = int(parts[1])
            seconds = float(parts[2])
            return hours * 3600 + minutes * 60 + seconds
    except ValueError:
        return None
    return None


def parse_csv_values(value: str) -> list[str]:
    """Split a comma-separated string into trimmed parts."""
    parts = [part.strip() for part in value.split(",")]
    return [part for part in parts if part]


def parse_misses(value: str | None) -> int | None:
    """Parse miss count from ShootingTotal-like strings."""
    if value is None:
        return None
    text = str(value)
    if not text:
        return None
    digits = re.findall(r"\d+", text)
    if not digits:
        return None
    return sum(int(d) for d in digits)


def parse_relay_shooting(value: str | None) -> tuple[int, int] | None:
    """Parse relay shooting format 'P+S' into (penalties, spares) tuple.

    In relays, format is 'penalties+spare_rounds' e.g. '2+3' means
    2 penalty loops and 3 spare rounds used.
    """
    if not value:
        return None
    text = str(value).strip()
    if "+" not in text:
        return None
    parts = text.split("+")
    if len(parts) != 2:
        return None
    try:
        return (int(parts[0]), int(parts[1]))
    except ValueError:
        return None


def parse_relay_shootings(value: str | None) -> tuple[tuple[int, int], tuple[int, int]] | None:
    """Parse relay Shootings format 'P+S P+S' into ((prone_p, prone_s), (stand_p, stand_s)).

    Returns tuple of (prone, standing) where each is (penalties, spares).
    """
    if not value:
        return None
    text = str(value).strip()
    parts = text.split()
    if len(parts) != 2:
        return None
    prone = parse_relay_shooting(parts[0])
    standing = parse_relay_shooting(parts[1])
    if prone is None or standing is None:
        return None
    return (prone, standing)


def format_relay_shooting(penalties: int, spares: int) -> str:
    """Format penalties and spares as 'P+S' string."""
    return f"{penalties}+{spares}"


def add_relay_shootings(
    shootings: list[tuple[int, int] | None],
) -> tuple[int, int]:
    """Sum multiple relay shooting tuples into total (penalties, spares)."""
    total_p, total_s = 0, 0
    for s in shootings:
        if s:
            total_p += s[0]
            total_s += s[1]
    return (total_p, total_s)


def is_dns(result: dict) -> bool:
    """Return True if result represents DNS (did not start) or similar."""
    irm = str(result.get("IRM") or "").upper()
    res_val = str(result.get("Result") or result.get("TotalTime") or "").upper()
    return irm == "DNS" or res_val == "DNS"


def get_first_time(result: dict, keys: list[str]) -> str:
    """Return the first non-empty time-like field from the result."""
    for key in keys:
        value = result.get(key)
        if value:
            return str(value)
    return ""


def extract_results(payload: dict) -> list[dict]:
    """Return sorted, non-team results list from a race payload."""
    candidates = [
        payload.get("Results"),
        payload.get("ResultList"),
        payload.get("Competitors"),
        payload.get("Data"),
    ]
    results = next((r for r in candidates if isinstance(r, list)), [])
    filtered = [res for res in results if not res.get("IsTeam")]
    filtered.sort(
        key=lambda res: (
            int(res.get("Rank")) if str(res.get("Rank", "")).isdigit() else 10**9,
            res.get("ResultOrder", 10**9),
        )
    )
    return filtered


def base_time_seconds(results: list[dict]) -> float | None:
    """Return the winner's absolute time in seconds if available."""
    for res in results:
        candidate = res.get("TotalTime") or res.get("Result")
        if candidate and not str(candidate).startswith("+"):
            parsed = parse_time_seconds(str(candidate))
            if parsed is not None:
                return parsed
    return None


def normalize_result_time(result: dict, base_seconds: float | None) -> str:
    """Convert result/total time to absolute clock if diff-based and base is known."""
    raw = get_first_time(result, ["Result", "TotalTime"]) or "-"
    if raw.startswith("+") and base_seconds is not None:
        diff = parse_time_seconds(raw)
        if diff is not None:
            return format_seconds(base_seconds + diff)
        return raw
    parsed = parse_time_seconds(raw)
    if parsed is not None:
        return format_seconds(parsed)
    return raw


def result_seconds(result: dict, base_seconds: float | None) -> float | None:
    """Return absolute time in seconds for a result row."""
    raw = get_first_time(result, ["Result", "TotalTime"])
    if not raw:
        return None
    if str(raw).upper() == "DNS":
        return None
    if str(raw).startswith("+") and base_seconds is not None:
        diff = parse_time_seconds(str(raw))
        if diff is None:
            return None
        return base_seconds + diff
    return parse_time_seconds(str(raw))


def sort_rows(rows: list[dict], column: str | None) -> list[dict]:
    """Sort rows by a given time column, pushing DNS/non-times to bottom."""
    if not column:
        return rows

    def key(row: dict) -> tuple:
        val = row.get(column)
        dns_flag = row.get("dns")
        if dns_flag:
            return (2, float("inf"), val)
        sec = parse_time_seconds(str(val)) if val not in ("", None, "-") else None
        if sec is None:
            return (1, float("inf"), val)
        return (0, sec, val)

    return sorted(rows, key=key)


def build_analytic_times(
    race_id: str,
    total_type: str,
    lap_prefix: str,
    lap_suffix: str,
    laps: int,
) -> dict:
    """Fetch analytic times per athlete keyed by IBUId/Bib."""
    times: dict = {}

    def merge(type_id: str, key: str) -> None:
        try:
            analytic = get_analytic_results(race_id, type_id)
        except BiathlonError:
            return
        for res in analytic.get("Results", []):
            if res.get("IsTeam"):
                continue
            ident = res.get("IBUId") or res.get("Bib") or res.get("Name")
            if not ident:
                continue
            times.setdefault(ident, {})[key] = get_first_time(res, ["TotalTime", "Result"])

    merge(total_type, "total")
    for idx in range(1, laps + 1):
        type_id = f"{lap_prefix}{idx}{lap_suffix}"
        merge(type_id, f"lap{idx}")
    return times


def get_race_label(race: dict) -> str:
    """Return a display label for a race."""
    return race.get("RaceName") or race.get("ShortDescription") or race.get("Description") or ""


def get_event_label(event: dict) -> str:
    """Return a display label for an event."""
    return event.get("ShortDescription") or event.get("Organizer") or ""


def get_race_start_key(race: dict) -> str:
    """Return a sortable start key for a race."""
    return race.get("StartTime") or race.get("StartDate") or race.get("FirstStart") or ""


def format_race_header(payload: dict, race_id: str) -> str:
    """Return a descriptive header for a race including location when available."""
    comp = payload.get("Competition") or {}
    sport_evt = payload.get("SportEvt") or {}

    race_label = comp.get("ShortDescription") or comp.get("Description") or race_id
    event_label = sport_evt.get("ShortDescription") or sport_evt.get("Organizer") or ""

    start = comp.get("StartTime") or ""
    date_part, time_part = "", ""
    if isinstance(start, str) and "T" in start:
        date_part, rest = start.split("T", 1)
        time_part = rest.rstrip("Z")
    location = f" — {event_label}" if event_label else ""
    return f"# {race_label}{location} {date_part} {time_part} ({race_id})".strip()


def format_result_row(result: dict, analytic_times: dict, base_secs: float | None) -> str:
    """Format a single competitor result with rank and finish time details."""
    rank = result.get("Rank") or result.get("ResultOrder") or ""
    name = result.get("Name") or result.get("ShortName") or ""
    result_time = normalize_result_time(result, base_secs)
    identifier = result.get("IBUId") or result.get("Bib") or name
    times = analytic_times.get(identifier, {})
    course_time = (
        times.get("course") or get_first_time(result, ["TotalCourseTime", "CourseTime", "RunTime"]) or "-"
    )
    range_time = times.get("range") or get_first_time(result, ["TotalRangeTime", "RangeTime"]) or "-"
    shooting_time = (
        times.get("shooting") or get_first_time(result, ["TotalShootingTime", "ShootingTime"]) or "-"
    )
    return f"{rank}\t{name}\t{result_time}\t{course_time}\t{range_time}\t{shooting_time}"


def resolve_race(race_arg: str, find_latest_func) -> tuple[str, dict]:
    """Return race id and payload, defaulting to latest race with results."""
    if race_arg:
        race_id = race_arg
        payload = get_race_results(race_id)
    else:
        race_id, payload = find_latest_func()
    return race_id, payload



"""Tests for constants module."""

from biathlon.constants import (
    RELAY_DISCIPLINE,
    INDIVIDUAL_DISCIPLINES,
    SHOTS_PER_DISCIPLINE,
    SKI_LAPS,
    SHOOTING_STAGES,
    GENDER_TO_CAT,
    CAT_TO_GENDER,
)


class TestDisciplineConstants:
    def test_relay_discipline(self):
        assert RELAY_DISCIPLINE == "RL"

    def test_individual_disciplines_is_set(self):
        assert isinstance(INDIVIDUAL_DISCIPLINES, set)

    def test_individual_disciplines_content(self):
        assert INDIVIDUAL_DISCIPLINES == {"SP", "PU", "IN", "MS", "SI"}

    def test_relay_not_in_individual(self):
        assert "RL" not in INDIVIDUAL_DISCIPLINES


class TestShotsPerDiscipline:
    def test_sprint_shots(self):
        assert SHOTS_PER_DISCIPLINE["SP"] == 10

    def test_pursuit_shots(self):
        assert SHOTS_PER_DISCIPLINE["PU"] == 20

    def test_individual_shots(self):
        assert SHOTS_PER_DISCIPLINE["IN"] == 20

    def test_mass_start_shots(self):
        assert SHOTS_PER_DISCIPLINE["MS"] == 20


class TestSkiLaps:
    def test_sprint_laps(self):
        assert SKI_LAPS["SP"] == 3

    def test_pursuit_laps(self):
        assert SKI_LAPS["PU"] == 5

    def test_individual_laps(self):
        assert SKI_LAPS["IN"] == 5

    def test_mass_start_laps(self):
        assert SKI_LAPS["MS"] == 5


class TestShootingStages:
    def test_sprint_stages(self):
        assert SHOOTING_STAGES["SP"] == 2

    def test_pursuit_stages(self):
        assert SHOOTING_STAGES["PU"] == 4

    def test_individual_stages(self):
        assert SHOOTING_STAGES["IN"] == 4

    def test_mass_start_stages(self):
        assert SHOOTING_STAGES["MS"] == 4


class TestGenderMappings:
    def test_gender_to_cat_women(self):
        assert GENDER_TO_CAT["women"] == "SW"

    def test_gender_to_cat_men(self):
        assert GENDER_TO_CAT["men"] == "SM"

    def test_cat_to_gender_sw(self):
        assert CAT_TO_GENDER["SW"] == "women"

    def test_cat_to_gender_sm(self):
        assert CAT_TO_GENDER["SM"] == "men"

    def test_mappings_are_inverse(self):
        for gender, cat in GENDER_TO_CAT.items():
            assert CAT_TO_GENDER[cat] == gender



"""Lightweight helper for calling the Biathlon results API."""

from __future__ import annotations

import json
import socket
from collections.abc import Iterable
from typing import Any
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import urlopen

API_BASE = "https://biathlonresults.com/modules/sportapi/api"


class BiathlonError(Exception):
    """Raised when the remote API call fails."""


def fetch_json(path: str) -> Iterable[dict[str, Any]]:
    """Retrieve JSON data from the API and decode it."""
    url = f"{API_BASE}/{path}"
    try:
        with urlopen(url, timeout=30) as resp:  # noqa: S310 - trusted domain
            if resp.status != 200:
                raise BiathlonError(f"Request failed: {resp.status} {resp.reason}")
            payload = resp.read()
    except HTTPError as exc:
        raise BiathlonError(f"Request failed: {exc.code} {exc.reason}") from exc
    except URLError as exc:
        raise BiathlonError(f"Network error: {exc.reason}") from exc
    except socket.timeout as exc:
        raise BiathlonError("Request timed out - server may be slow or unreachable") from exc

    try:
        return json.loads(payload.decode("utf-8"))
    except json.JSONDecodeError as exc:
        raise BiathlonError("Could not parse API response as JSON") from exc


def get_seasons() -> list[dict[str, Any]]:
    """Return all seasons from the API."""
    return list(fetch_json("Seasons"))


def get_current_season_id() -> str:
    """Return the current season id (fallback to the newest one)."""
    seasons = get_seasons()
    if not seasons:
        raise BiathlonError("No seasons returned by API")

    current = next((season for season in seasons if season.get("IsCurrent")), None)
    if current:
        return str(current.get("SeasonId"))

    newest = max(seasons, key=lambda season: season.get("SortOrder", 0))
    return str(newest.get("SeasonId"))


def get_events(season_id: str, level: int) -> list[dict[str, Any]]:
    """Return events for a season/level combination."""
    return list(fetch_json(f"Events?SeasonId={season_id}&Level={level}"))


def get_races(event_id: str) -> list[dict[str, Any]]:
    """Return races (competitions) for a given event."""
    return list(fetch_json(f"Competitions?EventId={event_id}"))


def get_race_results(race_id: str) -> dict[str, Any]:
    """Return full results payload for a race."""
    return dict(fetch_json(f"Results?RaceId={race_id}"))


def get_cups(season_id: str) -> list[dict[str, Any]]:
    """Return all cup definitions for a season."""
    return list(fetch_json(f"Cups?SeasonId={season_id}"))


def get_cup_results(cup_id: str) -> dict[str, Any]:
    """Return standings for a cup (score list)."""
    return dict(fetch_json(f"CupResults?CupId={cup_id}"))


def get_analytic_results(race_id: str, type_id: str) -> dict[str, Any]:
    """Return analytic results for a race (e.g., course/range/shooting times)."""
    return dict(fetch_json(f"AnalyticResults?RaceId={race_id}&TypeId={type_id}"))


def get_athlete_bio(ibu_id: str) -> dict[str, Any]:
    """Return CIS bio information for an athlete by IBU id."""
    return dict(fetch_json(f"CISBios?IBUId={ibu_id}"))


def get_athletes(family_name: str, given_name: str = "", request_id: int = 0) -> list[dict[str, Any]]:
    """Search athletes by family/given name."""
    query = urlencode({
        "FamilyName": family_name,
        "GivenName": given_name,
        "RequestId": request_id,
    })
    payload = fetch_json(f"Athletes?{query}")
    if isinstance(payload, dict):
        athletes = payload.get("Athletes") or payload.get("athletes") or []
        return list(athletes)
    return list(payload)


def get_all_results(ibu_id: str) -> dict[str, Any]:
    """Return results list for an athlete by IBU id."""
    return dict(fetch_json(f"AllResults?IBUId={ibu_id}"))



"""Command handlers for the Biathlon CLI."""

from .seasons import handle_seasons
from .events import handle_events
from .results import handle_results
from .cumulate import (
    handle_cumulate_results,
    handle_cumulate_ski,
    handle_cumulate_pursuit,
    handle_cumulate_course,
    handle_cumulate_range,
    handle_cumulate_shooting,
    handle_cumulate_miss,
    handle_cumulate_penalty,
    handle_cumulate_cleansheet,
    handle_cumulate_remontada,
)
from .standings import handle_standings
from .ceremony import handle_ceremony
from .athlete import handle_athlete_results, handle_athlete_results_scan, handle_athlete_info, handle_athlete_id
from .shooting import handle_shooting
from .relay import handle_relay
from .startlist import handle_startlist
from .post_race import handle_post_race
from .brief import handle_brief_event, handle_brief_startlist, handle_brief_post_race, handle_brief_season
from .form import handle_form

__all__ = [
    "handle_seasons",
    "handle_events",
    "handle_results",
    "handle_cumulate_results",
    "handle_cumulate_ski",
    "handle_cumulate_pursuit",
    "handle_cumulate_course",
    "handle_cumulate_range",
    "handle_cumulate_shooting",
    "handle_cumulate_miss",
    "handle_cumulate_penalty",
    "handle_cumulate_cleansheet",
    "handle_cumulate_remontada",
    "handle_standings",
    "handle_ceremony",
    "handle_athlete_results",
    "handle_athlete_results_scan",
    "handle_athlete_info",
    "handle_athlete_id",
    "handle_shooting",
    "handle_relay",
    "handle_startlist",
    "handle_post_race",
    "handle_brief_event",
    "handle_brief_season",
    "handle_brief_startlist",
    "handle_brief_post_race",
    "handle_form",
]



"""Tests for formatting functions."""

import argparse

import pytest

from biathlon.formatting import (
    format_seconds,
    format_pct,
    rank_style,
    is_pretty_output,
)


class TestFormatSeconds:
    def test_minutes_seconds(self):
        assert format_seconds(90.5) == "1:30.5"

    def test_hours_minutes_seconds(self):
        assert format_seconds(3661.5) == "1:01:01.5"

    def test_zero(self):
        assert format_seconds(0) == "0:00.0"

    def test_none(self):
        assert format_seconds(None) == "-"

    def test_sub_minute(self):
        assert format_seconds(45.3) == "0:45.3"


class TestFormatPct:
    def test_basic_percentage(self):
        assert format_pct(3, 4) == "75.0%"

    def test_full_percentage(self):
        assert format_pct(10, 10) == "100.0%"

    def test_zero_numerator(self):
        assert format_pct(0, 10) == "0.0%"

    def test_zero_denominator(self):
        assert format_pct(5, 0) == "-"


class TestRankStyle:
    def test_gold(self):
        assert rank_style(1) == "gold"

    def test_silver(self):
        assert rank_style(2) == "silver"

    def test_bronze(self):
        assert rank_style(3) == "bronze"

    def test_fourth(self):
        assert rank_style(4) == "flowers"

    def test_fifth(self):
        assert rank_style(5) == "flowers"

    def test_sixth(self):
        assert rank_style(6) == "flowers"

    def test_seventh(self):
        assert rank_style(7) == "other"

    def test_string_rank(self):
        assert rank_style("1") == "gold"

    def test_invalid_rank(self):
        assert rank_style("invalid") == "other"

    def test_none_rank(self):
        assert rank_style(None) == "other"


class TestIsPrettyOutput:
    def test_no_tsv_flag(self):
        args = argparse.Namespace()
        assert is_pretty_output(args) is True

    def test_tsv_false(self):
        args = argparse.Namespace(tsv=False)
        assert is_pretty_output(args) is True

    def test_tsv_true(self):
        args = argparse.Namespace(tsv=True)
        assert is_pretty_output(args) is False



"""Biathlon domain constants."""

# Discipline codes
RELAY_DISCIPLINE = "RL"
SINGLE_MIXED_RELAY_DISCIPLINE = "SR"
INDIVIDUAL_DISCIPLINES = {"SP", "PU", "IN", "MS", "SI"}
RELAY_DISCIPLINES = frozenset({"RL", "SR", "MR"})

# Relay category codes
RELAY_WOMEN_CAT = "SW"
RELAY_MEN_CAT = "SM"
RELAY_MIXED_CAT = "MX"

# Shots per discipline (5 shots per stage)
# Sprint: 2 stages, others: 4 stages
SHOTS_PER_DISCIPLINE = {"SP": 10, "PU": 20, "IN": 20, "MS": 20, "SI": 20}

# Skiing laps per discipline
SKI_LAPS = {"SP": 3, "PU": 5, "IN": 5, "MS": 5, "SI": 3}

# Shooting stages (range visits) per discipline
SHOOTING_STAGES = {"SP": 2, "PU": 4, "IN": 4, "MS": 4, "SI": 4}

# Gender/category mappings
GENDER_TO_CAT = {"women": "SW", "men": "SM"}
CAT_TO_GENDER = {"SW": "women", "SM": "men"}

# Discipline display names
DISCIPLINE_NAMES = {
    "SP": "Sprint",
    "PU": "Pursuit",
    "IN": "Individual",
    "MS": "Mass Start",
    "RL": "Relay",
    "SR": "Single Mixed Relay",
    "MR": "Mixed Relay",
    "SI": "Short Individual",
}

# Category display names for schedule tables
CATEGORY_DISPLAY_NAMES = {
    "SW": "Women",
    "SM": "Men",
    "MX": "Mixed",
}

# Event type constants
EVENT_TYPE_WC = "WC"
EVENT_TYPE_WCH = "WCH"
EVENT_TYPE_OWG = "OWG"

EVENT_TYPE_LABELS = {
    EVENT_TYPE_WC: "World Cup",
    EVENT_TYPE_WCH: "World Championship",
    EVENT_TYPE_OWG: "Olympic Games",
}



"""Shared helpers used across multiple command modules."""

from __future__ import annotations

import argparse
from concurrent.futures import ThreadPoolExecutor, as_completed
from typing import Any, Callable

from ..api import BiathlonError, get_analytic_results
from ..constants import EVENT_TYPE_OWG, EVENT_TYPE_WC, EVENT_TYPE_WCH, RELAY_DISCIPLINES
from ..formatting import Color, is_pretty_output
from ..utils import get_first_time


# Leader marker characters
GENERAL_LEADER_MARKER = "\u25CB"  # placeholder for yellow circle
DISCIPLINE_LEADER_MARKER = "\u25CC"  # placeholder for red circle
LEADER_MARKER_DOT = "\u25CF"


def _format_leader_markers(
    cell_str: str,
    row_idx: int,
    base_formatter: Callable[[str, int], str] | None = None,
) -> str:
    """Replace leader marker placeholders with colored dots.

    Extracts trailing GENERAL_LEADER_MARKER / DISCIPLINE_LEADER_MARKER tokens
    from *cell_str*, applies *base_formatter* to the remaining text, then
    appends gold/red filled dots.
    """
    text = cell_str.rstrip()
    pad_len = len(cell_str) - len(text)
    tokens = text.split()
    markers = []
    while tokens and tokens[-1] in {GENERAL_LEADER_MARKER, DISCIPLINE_LEADER_MARKER}:
        markers.insert(0, tokens.pop())
    base = " ".join(tokens)
    if base_formatter:
        base = base_formatter(base, row_idx)
    if markers:
        colored = []
        for marker in markers:
            if marker == GENERAL_LEADER_MARKER:
                colored.append(Color.gold(LEADER_MARKER_DOT))
            else:
                colored.append(Color.red(LEADER_MARKER_DOT))
        base = f"{base} {' '.join(colored)}" if base else " ".join(colored)
    return f"{base}{' ' * pad_len}"


def _row_ibu_id(row: dict) -> str:
    """Return the IBU id from a result/standings row."""
    for key in ("IBUId", "IbuId", "ibuId", "Id"):
        val = row.get(key)
        if val:
            return str(val)
    return ""


def _parse_rank(value: Any) -> int | None:
    """Parse a rank value, stripping trailing dots."""
    text = str(value).strip().rstrip(".")
    if text.isdigit():
        return int(text)
    return None


def _format_section_title(text: str, args: argparse.Namespace) -> str:
    """Return *text* styled as a section title when pretty output is on."""
    if not is_pretty_output(args):
        return text
    return Color.section_title(text)


def _ordinal(n: int) -> str:
    """Convert a number to its ordinal string (1st, 2nd, 3rd, etc.)."""
    if 11 <= n % 100 <= 13:
        return f"{n}th"
    return f"{n}{['th', 'st', 'nd', 'rd', 'th'][min(n % 10, 4)]}"


def _max_workers(total: int, cap: int = 15) -> int:
    """Return a capped worker count for concurrent fetches."""
    return min(cap, max(1, total))


def detect_event_type(event: dict) -> str:
    """Detect event type from event description.

    Returns:
        EVENT_TYPE_OWG for Olympic Games
        EVENT_TYPE_WCH for World Championships
        EVENT_TYPE_WC for regular World Cup
    """
    desc = str(event.get("Description") or event.get("ShortDescription") or "").lower()
    if "olympic" in desc:
        return EVENT_TYPE_OWG
    if "world championships" in desc:
        return EVENT_TYPE_WCH
    return EVENT_TYPE_WC


def is_relay_discipline(discipline: str) -> bool:
    """Return True if *discipline* is any relay type."""
    return discipline in RELAY_DISCIPLINES


def is_mixed_relay(discipline: str, category: str) -> bool:
    """Return True if the race is a mixed relay."""
    if discipline in {"MR", "SR"}:
        return True
    return discipline == "RL" and category == "MX"


# ---------------------------------------------------------------------------
# Analytics helpers shared by cumulate and shooting commands
# ---------------------------------------------------------------------------


def _parse_shootings(value: str | None) -> list[int]:
    """Parse shootings string like '0+1+0+2' into list of ints."""
    if not value:
        return []
    parts = [p.strip() for p in str(value).split("+") if p.strip()]
    misses: list[int] = []
    for part in parts:
        try:
            misses.append(int(part))
        except ValueError:
            misses.append(0)
    return misses


def _fetch_analytic_map(race_id: str, type_id: str) -> dict[str, str]:
    """Fetch analytic times keyed by IBUId/Bib/Name."""
    try:
        analytic = get_analytic_results(race_id, type_id)
    except BiathlonError:
        return {}
    times: dict[str, str] = {}
    for res in analytic.get("Results", []):
        if res.get("IsTeam"):
            continue
        ident = res.get("IBUId") or res.get("Bib") or res.get("Name")
        if not ident:
            continue
        times[str(ident)] = get_first_time(res, ["TotalTime", "Result"]) or "-"
    return times


def _lookup_analytic_time(times: dict[str, str], res: dict) -> str:
    """Return analytic time for a result using multiple identifier keys."""
    for key in (res.get("IBUId"), res.get("Bib"), res.get("Name"), res.get("ShortName")):
        if key and str(key) in times:
            return times[str(key)]
    return ""


def _prefetch_analytic_maps(
    requests: list[tuple[str, str]],
) -> dict[tuple[str, str], dict[str, str]]:
    """Fetch multiple analytic maps in parallel.

    Args:
        requests: List of (race_id, type_id) tuples to fetch.

    Returns:
        Dict mapping (race_id, type_id) to the analytic map result.
    """
    if not requests:
        return {}
    results: dict[tuple[str, str], dict[str, str]] = {}
    if len(requests) == 1:
        race_id, type_id = requests[0]
        results[(race_id, type_id)] = _fetch_analytic_map(race_id, type_id)
        return results
    with ThreadPoolExecutor(max_workers=_max_workers(len(requests), cap=8)) as executor:
        futures = {
            executor.submit(_fetch_analytic_map, race_id, type_id): (race_id, type_id)
            for race_id, type_id in requests
        }
        for future in as_completed(futures):
            key = futures[future]
            try:
                results[key] = future.result()
            except Exception:
                results[key] = {}
    return results


def _fetch_leg_lap_times(
    race_id: str,
    lap_prefix: str,
    lap_suffix: str,
    laps: int,
    laps_per_leg: int,
) -> dict[tuple[str, int], dict[str, str]]:
    """Fetch analytic lap times keyed by (Bib/IBUId/Name, Leg) -> {lapN: time_str}."""
    times: dict[tuple[str, int], dict[str, str]] = {}
    for idx in range(1, laps + 1):
        type_id = f"{lap_prefix}{idx}{lap_suffix}"
        try:
            analytic = get_analytic_results(race_id, type_id)
        except BiathlonError:
            continue
        for res in analytic.get("Results", []):
            if res.get("IsTeam"):
                continue
            bib = str(res.get("Bib") or "")
            ibu_id = str(res.get("IBUId") or "")
            name = str(res.get("Name") or "")
            leg = res.get("Leg")
            if leg is None:
                leg_idx = (idx - 1) // laps_per_leg + 1
                local_idx = (idx - 1) % laps_per_leg + 1
            else:
                leg_idx = int(leg)
                local_idx = idx - (leg_idx - 1) * laps_per_leg
                if local_idx < 1 or local_idx > laps_per_leg:
                    local_idx = (idx - 1) % laps_per_leg + 1
            time_str = get_first_time(res, ["TotalTime", "Result"])
            if time_str:
                if bib:
                    times.setdefault((bib, leg_idx), {})[f"lap{local_idx}"] = time_str
                if ibu_id:
                    times.setdefault((ibu_id, leg_idx), {})[f"lap{local_idx}"] = time_str
                if name:
                    times.setdefault((name, leg_idx), {})[f"lap{local_idx}"] = time_str
    return times



"""CLI entry point for Biathlon results."""

from __future__ import annotations

import argparse
import textwrap
import sys
from collections.abc import Iterable
from importlib.metadata import version, PackageNotFoundError

from .api import BiathlonError


def get_version() -> str:
    """Get package version."""
    try:
        return version("biathlon")
    except PackageNotFoundError:
        return "dev"


from .commands import (
    handle_athlete_id,
    handle_athlete_info,
    handle_athlete_results,
    handle_brief_event,
    handle_brief_post_race,
    handle_brief_season,
    handle_brief_startlist,
    handle_ceremony,
    handle_cumulate_results,
    handle_cumulate_ski,
    handle_cumulate_pursuit,
    handle_cumulate_course,
    handle_cumulate_range,
    handle_cumulate_shooting,
    handle_cumulate_miss,
    handle_cumulate_penalty,
    handle_cumulate_cleansheet,
    handle_cumulate_remontada,
    handle_events,
    handle_form,
    handle_results,
    handle_standings,
    handle_seasons,
    handle_shooting,
)


class CompactOptionalFormatter(argparse.RawTextHelpFormatter):
    """Formatter that groups optional flags before their metavar."""

    def __init__(self, prog: str, indent_increment: int = 2, max_help_position: int = 40, width: int | None = None) -> None:
        super().__init__(prog, indent_increment, max_help_position, width)

    def _format_action_invocation(self, action: argparse.Action) -> str:
        if not action.option_strings:
            return super()._format_action_invocation(action)
        opts = ", ".join(action.option_strings)
        if action.nargs != 0:
            opts += f" {self._format_args(action, action.dest.upper())}"
        return opts

    def _format_action(self, action: argparse.Action) -> str:
        if isinstance(action, argparse._SubParsersAction):
            parts = []
            subactions = list(action._get_subactions())
            self._indent()
            for subaction in subactions:
                if not subaction.help:
                    continue
                parts.append(super()._format_action(subaction))
            self._dedent()
            return "".join(parts)
        return super()._format_action(action)


def traverse_to_parser(
    parser: argparse.ArgumentParser,
    tokens: list[str],
) -> tuple[argparse.ArgumentParser, list[str]]:
    """Traverse subparsers to the deepest matching parser."""
    if not parser._subparsers:
        return parser, tokens
    subparsers_action = None
    for action in parser._subparsers._actions:
        if isinstance(action, argparse._SubParsersAction):
            subparsers_action = action
            break
    if not subparsers_action or not tokens:
        return parser, tokens
    command = tokens[0]
    choices = subparsers_action.choices
    if command not in choices:
        return parser, tokens
    return traverse_to_parser(choices[command], tokens[1:])



def add_output_flag(subparser: argparse.ArgumentParser) -> None:
    """Add --tsv flag to a subparser."""
    subparser.add_argument(
        "--tsv",
        action="store_true",
        help="Output TSV instead of aligned table",
    )


def add_cumulate_args(subparser: argparse.ArgumentParser, allow_discipline_event: bool) -> None:
    """Add common cumulate arguments to a subparser."""
    subparser.add_argument(
        "--men",
        action="store_true",
        help="Show men (default: women)",
    )
    if allow_discipline_event:
        subparser.add_argument(
            "--discipline",
           
            default="all",
            choices=[
                "individual",
                "sprint",
                "pursuit",
                "mass-start",
                "relay",
                "mixed-relay",
                "single-mixed-relay",
                "all",
            ],
            metavar="DISCIPLINE",
            help="Race discipline (default: all)",
        )
        event_help = "Event id (overrides --discipline and --season)"
    else:
        event_help = "Event id (only supported for cumulate results)"
    subparser.add_argument(
        "--event",
       
        default="",
        help=event_help,
    )
    subparser.add_argument(
        "--season",
       
        default="",
        help="Season id (default: current season)",
    )
    subparser.add_argument(
        "--include-relay",
        action="store_true",
        help="Include relay races in cumulate calculations",
    )
    subparser.add_argument(
        "--top",
       
        type=int,
        default=0,
        help="Filter to top N athletes in WC standings",
    )
    subparser.add_argument(
        "--limit",
       
        type=int,
        default=25,
        help="Number of rows to display (default: 25, 0 for all)",
    )
    add_output_flag(subparser)


def build_parser() -> argparse.ArgumentParser:
    """Build the main argument parser with all subcommands."""
    parser = argparse.ArgumentParser(
        prog="biathlon",
        description="CLI for exploring IBU biathlon results stored on biathlonresults.com",
        usage="\n    biathlon command [subcommand] [parameters]",
        add_help=False,
        formatter_class=CompactOptionalFormatter,
        epilog=textwrap.dedent("""\
            Examples:
                biathlon events --races           Get events and races for the current season
                biathlon results --men --detail   Get detailed results for the most recent men race
                biathlon cumulate remontada       Get women biathlete with biggest pursuit gains this season
                biathlon standings help           Get help for standings command
        """),
    )
    parser._positionals.title = "Available commands"
    subparsers = parser.add_subparsers(dest="command")

    # --- seasons ---
    seasons_parser = subparsers.add_parser(
        "seasons",
        help="List seasons available",
        usage="\n  biathlon seasons [parameters]",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    seasons_parser._optionals.title = "optional parameters"
    seasons_parser.add_argument("--limit", metavar="INT", type=int, default=25, help="Limit output rows (default: 25, 0 for all)")
    add_output_flag(seasons_parser)
    seasons_parser.set_defaults(func=handle_seasons)

    # --- events ---
    events_parser = subparsers.add_parser(
        "events",
        help="List season events",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    events_parser._optionals.title = "optional parameters"
    events_parser.add_argument("--season", default="", help="Select specific season Id or 'all' (default: current season)")
    events_parser.add_argument(
        "--level", default="1",
        help="Select specific competitien levels: -1=All, 0=Mixed, 1=World Cup (default), 2=IBU Cup, 3=Junior, 4=Other, 5=Regional, 6=Para",
    )
    events_parser.add_argument("--search", default="", help="Filter events by name")
    events_parser.add_argument("--sort", default="startdate", help="Sort order")
    events_parser.add_argument("--completed", action="store_true", help="Only completed events")
    events_parser.add_argument("--upcoming", action="store_true", help="Only current/next and upcoming events")
    events_parser.add_argument("--summary", action="store_true", help="Show race-type availability per event")
    events_parser.add_argument("--races", action="store_true", help="Include races under each event")
    events_parser.add_argument(
        "--discipline", default="", 
        help="Filter races by discipline (individual, sprint, pursuit, mass-start, relay, mixed-relay, single-mixed-relay)"
    )
    add_output_flag(events_parser)
    events_parser.set_defaults(func=handle_events)

    # --- results ---
    results_parser = subparsers.add_parser(
        "results",
        help="Show race results",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    results_parser.add_argument("--race", default="", help="Race id (default: most recent race)")
    results_parser.add_argument("--men", action="store_true", help="Show men (default: women)")
    results_parser.add_argument(
        "--discipline",
       
        default="",
        help="Discipline filter (mutually exclusive with --race)",
    )
    results_parser.add_argument("--detail", action="store_true", help="Show detailed split columns")
    results_parser.add_argument("--sort", default="", help="Sort by column")
    results_parser.add_argument("--country", default="", metavar="COUNTRY", help="Filter by country code (e.g., FRA, GER, NOR)")
    results_parser.add_argument("--top", type=int, default=0, help="Filter to top N athletes in World Cup standings")
    results_parser.add_argument("--first", type=int, default=0, help="Filter to first N finishers in the race")
    results_parser.add_argument("--limit", type=int, default=25, help="Limit output rows (default: 25, 0 for all)")
    results_parser.add_argument(
        "--highlight-wc",
        action="store_true",
        help="Highlight top 6 by World Cup standing instead of race rank",
    )
    add_output_flag(results_parser)
    results_parser.set_defaults(func=handle_results)

    # --- cumulate ---
    cumulate_parser = subparsers.add_parser(
        "cumulate",
        help="Show cumulative rankings (results, course, miss, position gain, etc.)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )

    cumulate_help: dict[str, str] = {}

    def _show_cumulate_help(_args: argparse.Namespace) -> int:
        print("usage: biathlon cumulate <subcommand> [options]\n")
        print("subcommands:")
        width = max((len(name) for name in cumulate_sub.choices.keys()), default=0)
        for name in cumulate_sub.choices.keys():
            help_text = cumulate_help.get(name, "")
            print(f"  {name.ljust(width)}  {help_text}")
        return 0

    add_cumulate_args(cumulate_parser, allow_discipline_event=True)
    cumulate_parser._custom_help = _show_cumulate_help
    cumulate_parser.set_defaults(func=_show_cumulate_help, cumulate_command=None)
    cumulate_sub = cumulate_parser.add_subparsers(dest="cumulate_command", title="subcommands", metavar="")

    cumulate_results = cumulate_sub.add_parser(
        "results", help="Cumulated results", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["results"] = "Cumulated race results"
    add_cumulate_args(cumulate_results, allow_discipline_event=True)
    cumulate_results.set_defaults(func=handle_cumulate_results)

    cumulate_ski = cumulate_sub.add_parser(
        "ski", help="Cumulated ski times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["ski"] = "Cumulated ski times (individual races only ; results without penalties)"
    add_cumulate_args(cumulate_ski, allow_discipline_event=False)
    cumulate_ski.set_defaults(func=handle_cumulate_ski)

    cumulate_pursuit = cumulate_sub.add_parser(
        "pursuit", help="Cumulated pursuit times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["pursuit"] = "Cumulated pursuit times (pursuit races only ; results without start delay)"
    add_cumulate_args(cumulate_pursuit, allow_discipline_event=False)
    cumulate_pursuit.set_defaults(func=handle_cumulate_pursuit)

    cumulate_course = cumulate_sub.add_parser(
        "course", help="Cumulated course times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["course"] = "Cumulated course times (ski time only ; range, start delay, penalties excluded)"
    add_cumulate_args(cumulate_course, allow_discipline_event=True)
    cumulate_course.set_defaults(func=handle_cumulate_course)

    cumulate_range = cumulate_sub.add_parser(
        "range", help="Cumulated range times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["range"] = "Cumulated range times"
    add_cumulate_args(cumulate_range, allow_discipline_event=True)
    cumulate_range.set_defaults(func=handle_cumulate_range)

    cumulate_shooting = cumulate_sub.add_parser(
        "shooting", help="Cumulated shooting times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["shooting"] = "Cumulated shooting times"
    add_cumulate_args(cumulate_shooting, allow_discipline_event=True)
    cumulate_shooting.set_defaults(func=handle_cumulate_shooting)

    cumulate_miss = cumulate_sub.add_parser(
        "miss", help="Cumulated misses", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["miss"] = "Cumulated misses"
    add_cumulate_args(cumulate_miss, allow_discipline_event=True)
    cumulate_miss.set_defaults(func=handle_cumulate_miss)

    cumulate_penalty = cumulate_sub.add_parser(
        "penalty", help="Cumulated penalty times", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["penalty"] = "Cumulated penalty times"
    add_cumulate_args(cumulate_penalty, allow_discipline_event=True)
    cumulate_penalty.set_defaults(func=handle_cumulate_penalty)

    cumulate_remontada = cumulate_sub.add_parser(
        "remontada", help="Cumulated pursuit gains", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["remontada"] = "Cumulated pursuit gains"
    add_cumulate_args(cumulate_remontada, allow_discipline_event=False)
    cumulate_remontada.set_defaults(func=handle_cumulate_remontada)

    cumulate_cleansheet = cumulate_sub.add_parser(
        "cleansheet", help="Cumulated clean shooting stages (5/5)", formatter_class=CompactOptionalFormatter, add_help=False
    )
    cumulate_help["cleansheet"] = "Cumulated clean shooting stages (5/5)"
    add_cumulate_args(cumulate_cleansheet, allow_discipline_event=True)
    cumulate_cleansheet.add_argument(
        "--sort",
        default="cleansheets",
        choices=["cleansheets", "percentage", "time"],
        help="Sort by column (default: cleansheets)",
    )
    cumulate_cleansheet.add_argument(
        "--min",
        type=int,
        default=66,
        dest="min_pct",
        metavar="PCT",
        help="Minimum race participation percentage (0-100, default: 50)",
    )
    cumulate_cleansheet.set_defaults(func=handle_cumulate_cleansheet)

    # --- standings ---
    standings_parser = subparsers.add_parser("standings", help="Show standings (world cup, IBU Cup, etc.)", formatter_class=CompactOptionalFormatter, add_help=False)
    standings_parser.add_argument("--season", default="", help="Season id")
    standings_parser.add_argument("--men", action="store_true", help="Show men")
    standings_parser.add_argument("--level", default="1", help="Cup level")
    standings_parser.add_argument("--sort", default="total", choices=["total", "sprint", "pursuit", "individual", "massstart"], help="Sort by column")
    standings_parser.add_argument("--limit", type=int, default=25, help="Limit output rows (default: 25, 0 for all)")
    add_output_flag(standings_parser)
    standings_parser.set_defaults(func=handle_standings)

    # --- ceremony ---
    ceremony_parser = subparsers.add_parser("ceremony", help="Show medal standing (default: men+women, World Cup, current season)", formatter_class=CompactOptionalFormatter, add_help=False)
    ceremony_parser.add_argument("--athlete", action="store_true", help="Rank by athlete (default: by country)")
    ceremony_parser.add_argument("--race", default="", help="Race id")
    ceremony_parser.add_argument("--event", default="", help="Event id")
    gender_group = ceremony_parser.add_mutually_exclusive_group()
    gender_group.add_argument("--men", action="store_true", help="Show men only (default: men+women)")
    gender_group.add_argument("--women", action="store_true", help="Show women only (default: men+women)")
    ceremony_parser.add_argument("--country", default="", help="Filter by host country (where event is held)")
    ceremony_parser.add_argument("--search", default="", help="Filter events by name (e.g., 'annecy', 'holmenkollen')")
    ceremony_parser.add_argument("--season", default="", help="Season id (default: current season)")
    add_output_flag(ceremony_parser)
    ceremony_parser.set_defaults(func=handle_ceremony)

    # --- shooting ---
    shooting_parser = subparsers.add_parser("shooting", help="Show shooting accuracy", formatter_class=CompactOptionalFormatter, add_help=False)
    shooting_parser.add_argument("--race", default="", help="Race id")
    shooting_parser.add_argument("--event", default="", help="Event id")
    shooting_parser.add_argument("--season", default="", help="Season id")
    shooting_parser.add_argument("--men", action="store_true", help="Show men")
    shooting_parser.add_argument(
        "--include-relay",
        default="",
        choices=["relay", "mixed-relay", "single-mixed", "all", ""],
        help="Include relay races in shooting stats (relay, mixed-relay, single-mixed, all)",
    )
    shooting_parser.add_argument("--all-races", action="store_true", help="Only athletes who started every race")
    shooting_parser.add_argument("--sort", default="", help="Sort order")
    shooting_parser.add_argument("--min", type=int, default=50, dest="min_pct", help="Minimum race participation %% (default: 50)")
    shooting_parser.add_argument("--top", type=int, default=0, help="Restrict to top N athletes in WC standings")
    shooting_parser.add_argument("--limit", type=int, default=25, help="Limit output rows (default: 25, 0 for all)")
    shooting_parser.add_argument("--debug-races", action="store_true", help="Debug: print races considered")
    add_output_flag(shooting_parser)
    shooting_parser.set_defaults(func=handle_shooting)

    # --- athlete ---
    athlete_parser = subparsers.add_parser("athlete", help="Show athlete information", formatter_class=CompactOptionalFormatter, add_help=False)
    athlete_parser.add_argument("--id", default="", help="Athlete IBU id (comma-separated)")
    athlete_parser.add_argument("--search", default="", help="Search by name")
    athlete_parser.add_argument("--season", default="", help="Season id")
    add_output_flag(athlete_parser)
    athlete_parser.set_defaults(func=handle_athlete_info, athlete_command=None)
    athlete_sub = athlete_parser.add_subparsers(dest="athlete_command", title="subcommands", metavar="")

    athlete_results = athlete_sub.add_parser("results", help="Season race ranks (AllResults)", formatter_class=CompactOptionalFormatter, add_help=False)
    athlete_results.add_argument("--id", default="", help="Athlete IBU id")
    athlete_results.add_argument("--season", default="", help="Season id or label (e.g., 2526 or 25/26)")
    athlete_results.add_argument("--level", default="WC,OWG,WCH", help="Level filter (default: WC,OWG,WCH — use 'all' for all levels)")
    athlete_results.add_argument("--course", action="store_true", help="Use course time rank")
    add_output_flag(athlete_results)
    athlete_results.set_defaults(func=handle_athlete_results)

    athlete_info = athlete_sub.add_parser("info", help="Athlete bio info", formatter_class=CompactOptionalFormatter, add_help=False)
    athlete_info.add_argument("--id", default="", help="Athlete IBU id (comma-separated)")
    athlete_info.add_argument("--search", default="", help="Search by name")
    athlete_info.add_argument("--season", default="", help="Season id")
    athlete_info.add_argument("--level", type=int, default=0, help="Event level (1-5, 0 for all)")
    add_output_flag(athlete_info)
    athlete_info.set_defaults(func=handle_athlete_info)

    athlete_id = athlete_sub.add_parser("id", help="Find athlete IBU ids", formatter_class=CompactOptionalFormatter, add_help=False)
    athlete_id.add_argument("--search", default="", help="Search by name")
    athlete_id.add_argument("--season", default="", help="Season id")
    athlete_id.add_argument("--level", type=int, default=0, help="Event level (1-5, 0 for all)")
    add_output_flag(athlete_id)
    athlete_id.set_defaults(func=handle_athlete_id)

    # --- brief ---
    brief_parser = subparsers.add_parser(
        "brief",
        help="Race analysis (event, season, startlist, postrace)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    brief_help: dict[str, str] = {}

    def _show_brief_help(_args: argparse.Namespace) -> int:
        print("usage: biathlon brief <subcommand> [options]\n")
        print("subcommands:")
        width = max((len(name) for name in brief_sub.choices.keys()), default=0)
        for name in brief_sub.choices.keys():
            help_text = brief_help.get(name, "")
            print(f"  {name.ljust(width)}  {help_text}")
        return 0

    brief_parser._custom_help = _show_brief_help
    brief_parser.set_defaults(func=_show_brief_help, brief_command=None)
    brief_sub = brief_parser.add_subparsers(dest="brief_command", title="subcommands", metavar="")

    # brief event
    brief_event = brief_sub.add_parser(
        "event",
        help="Venue history and records (before an event)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    brief_help["event"] = "Venue history and records (before an event)"
    brief_event.add_argument("--event", default="", help="Event id (default: current/upcoming WC event)")
    brief_event.add_argument("--men", action="store_true", help="Show men (default: women)")
    brief_event.add_argument("--major", action="store_true", help="Use WC+WCH+OWG stats")
    add_output_flag(brief_event)
    brief_event.set_defaults(func=handle_brief_event)

    # brief season
    brief_season = brief_sub.add_parser(
        "season",
        help="Season summary",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    brief_help["season"] = "Season summary (events and race counts)"
    brief_season.add_argument("--season", default="", help="Season id (default: current season)")
    brief_season.add_argument("--level", default="1", help="Event level (default: 1)")
    add_output_flag(brief_season)
    brief_season.set_defaults(func=handle_brief_season)

    # brief startlist
    brief_startlist = brief_sub.add_parser(
        "startlist",
        help="Startlist analysis (before a race)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    brief_help["startlist"] = "Startlist analysis (before a race)"
    brief_startlist.add_argument("--race", default="", help="Race id (default: latest WC startlist)")
    brief_startlist.add_argument("--major", action="store_true", help="Use WC+WCH+OWG milestones")
    add_output_flag(brief_startlist)
    brief_startlist.set_defaults(func=handle_brief_startlist)

    # brief postrace
    brief_post_race = brief_sub.add_parser(
        "postrace",
        help="Post-race analysis (after a race)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    brief_help["postrace"] = "Post-race analysis (after a race)"
    brief_post_race.add_argument("--race", default="", help="Race id (default: latest completed race)")
    brief_post_race.add_argument("--major", action="store_true", help="Use WC+WCH+OWG milestones")
    add_output_flag(brief_post_race)
    brief_post_race.set_defaults(func=handle_brief_post_race)

    # --- form ---
    form_parser = subparsers.add_parser(
        "form",
        help="Show recent athlete form (course time ranks)",
        formatter_class=CompactOptionalFormatter,
        add_help=False,
    )
    form_parser._optionals.title = "optional parameters"
    form_parser.add_argument("--men", action="store_true", help="Show men (default: women)")
    form_parser.add_argument("--startlist", nargs="?", const="", default=None, help="Filter to athletes from a race startlist (auto-detects gender). Without a race id, discovers available startlists.")
    form_parser.add_argument("--races", type=int, default=5, help="Number of recent races for current form (default: 5)")
    form_parser.add_argument(
        "--event",
        type=int,
        default=0,
        metavar="N",
        help="Current form from last N events (mutually exclusive with --races)",
    )
    form_parser.add_argument("--limit", type=int, default=25, help="Limit output rows (default: 25, 0 for all)")
    form_parser.add_argument("--top", type=int, default=0, help="Filter to top N in WC standings")
    form_parser.add_argument("--shoot", action="store_true", help="Show shooting accuracy instead of course time ranks")
    form_parser.add_argument("--min", type=int, default=50, dest="min_pct", metavar="PCT", help="Minimum race participation percentage (0-100, default: 50)")
    form_parser.add_argument("--season", action="store_true", help="Calculate form based on all season races")
    form_parser.add_argument(
        "--remove",
        action="append",
        default=[],
        metavar="DISC",
        help="Remove discipline from calculations (sprint, pursuit, individual, mass-start). Can be repeated.",
    )
    form_parser.add_argument(
        "--include-relay",
        default="",
        choices=["relay", "mixed-relay", "single-mixed", "all", ""],
        help="Include relay races (relay, mixed-relay, single-mixed, all)",
    )
    add_output_flag(form_parser)
    form_parser.set_defaults(func=handle_form)

    return parser


def main(argv: Iterable[str] | None = None) -> int:
    """Main CLI entry point."""
    tokens = list(argv) if argv is not None else sys.argv[1:]

    # Handle --version before parsing
    if tokens and tokens[0] in ("--version", "-V"):
        print(f"biathlon {get_version()}")
        return 0


    parser = build_parser()

    if tokens and tokens[-1] == "help":
        target_tokens = tokens[:-1]
        target_parser, remaining = traverse_to_parser(parser, target_tokens)
        if remaining:
            print(f"error: unknown command {' '.join(remaining)}", file=sys.stderr)
            return 1
        custom_help = getattr(target_parser, "_custom_help", None)
        if custom_help:
            print()
            return custom_help(argparse.Namespace())
        print()
        target_parser.print_help()
        print()
        return 0

    args = parser.parse_args(tokens)

    if args.command is None:
        print(
            "\nbiathlon: [ERROR]: the following arguments are required: <command>\n\n"
            "Usage: biathlon <command> [<subcommand>] [parameters]\n\n"
            "Example: biathlon events --races\n\n"
            "To see help text, you can run:\n"
            "  biathlon help\n"
            "  biathlon <command> help\n"
            "  biathlon <command> <subcommand> help\n",
            file=sys.stderr,
        )
        return 2

    if _require_subcommand(args):
        return 2

    try:
        return args.func(args)
    except KeyboardInterrupt:
        print()  # Print newline to clean up partial output
        return 130
    except BrokenPipeError:
        return 0
    except BiathlonError as exc:
        print(f"error: {exc}", file=sys.stderr)
        return 1


def _require_subcommand(args: argparse.Namespace) -> bool:
    """Return True if a required subcommand is missing."""
    if args.command == "cumulate" and getattr(args, "cumulate_command", None) is None:
        print(
            "\nbiathlon cumulate: [ERROR]: the following arguments are required: <subcommand>\n\n"
            "Usage: biathlon cumulate <subcommand> [parameters]\n\n"
            "Example: biathlon cumulate results --detail\n\n"
            "To see help text, you can run:\n"
            "  biathlon cumulate help\n"
            "  biathlon cumulate <subcommand> help\n",
            file=sys.stderr,
        )
        return True
    if args.command == "athlete" and getattr(args, "athlete_command", None) is None:
        print(
            "\nbiathlon athlete: [ERROR]: the following arguments are required: <subcommand>\n\n"
            "Usage: biathlon athlete <subcommand> [parameters]\n\n"
            "Example: biathlon athlete info --search \"Boe\"\n\n"
            "To see help text, you can run:\n"
            "  biathlon athlete help\n"
            "  biathlon athlete <subcommand> help\n",
            file=sys.stderr,
        )
        return True
    if args.command == "brief" and getattr(args, "brief_command", None) is None:
        print(
            "\nbiathlon brief: [ERROR]: the following arguments are required: <subcommand>\n\n"
            "Usage: biathlon brief <subcommand> [parameters]\n\n"
            "Example: biathlon brief startlist\n\n"
            "To see help text, you can run:\n"
            "  biathlon brief help\n"
            "  biathlon brief <subcommand> help\n",
            file=sys.stderr,
        )
        return True
    return False


if __name__ == "__main__":  # pragma: no cover
    raise SystemExit(main())



"""Results command handler (replacement for results + relay)."""

from __future__ import annotations

import argparse
import datetime
import sys

from ..api import (
    BiathlonError,
    get_analytic_results,
    get_cup_results,
    get_current_season_id,
    get_events,
    get_race_results,
    get_races,
)
from ..constants import (
    CAT_TO_GENDER,
    RELAY_DISCIPLINE,
    RELAY_MEN_CAT,
    RELAY_MIXED_CAT,
    RELAY_WOMEN_CAT,
    SINGLE_MIXED_RELAY_DISCIPLINE,
    SKI_LAPS,
    SHOOTING_STAGES,
)
from ..formatting import format_seconds, is_pretty_output, rank_style, render_table
from ..utils import (
    add_relay_shootings,
    base_time_seconds,
    extract_results,
    format_race_header,
    format_relay_shooting,
    get_first_time,
    get_race_start_key,
    is_dns,
    normalize_result_time,
    parse_relay_shooting,
    parse_relay_shootings,
    parse_start_datetime,
    parse_time_seconds,
)
from ._common import _row_ibu_id
from .relay import _has_completed_results as _has_completed_relay_results
from .standings import find_cup_id


def _get_wc_rows(cat_id: str, season_id: str) -> list[dict]:
    """Fetch World Cup standings rows for a category/season."""
    gender = CAT_TO_GENDER.get(cat_id.upper())
    if not gender:
        return []
    try:
        cup_id = find_cup_id(season_id, gender, level=1, cup_type="total")
        payload = get_cup_results(cup_id)
        return payload.get("Rows") or payload.get("Results") or []
    except BiathlonError:
        return []


def _get_wc_rank_map(cat_id: str, top_n: int, season_id: str | None = None) -> dict[str, int]:
    """Return IBU id -> WC rank mapping for the top N."""
    if top_n <= 0:
        return {}
    season = season_id or get_current_season_id()
    rows = _get_wc_rows(cat_id, season)
    rank_map: dict[str, int] = {}
    for row in rows:
        ibu_id = _row_ibu_id(row)
        if not ibu_id:
            continue
        try:
            rank = int(row.get("Rank") or row.get("rank") or 0)
        except (TypeError, ValueError):
            continue
        if rank <= 0:
            continue
        rank_map[ibu_id] = rank
        if len(rank_map) >= top_n:
            break
    return rank_map


def _get_top_n_ibu_ids(cat_id: str, top_n: int, season_id: str | None = None) -> list[str]:
    """Return IBU ids for the top N WC standings."""
    if top_n <= 0:
        return []
    season = season_id or get_current_season_id()
    rows = _get_wc_rows(cat_id, season)
    ids: list[str] = []
    for row in rows:
        ibu_id = _row_ibu_id(row)
        if not ibu_id:
            continue
        ids.append(ibu_id)
        if len(ids) >= top_n:
            break
    return ids


def _has_completed_results(payload: dict) -> bool:
    """Return True when a race payload contains completed results."""
    results = payload.get("Results", [])
    if not results:
        return False
    for res in results:
        if res.get("IsTeam"):
            continue
        rank = res.get("Rank")
        if rank is not None:
            rank_text = str(rank).strip()
            if rank_text and rank_text != "10000":
                return True
        result_val = res.get("Result") or res.get("TotalTime")
        if result_val:
            result_text = str(result_val).strip().upper()
            if result_text and result_text not in {"DNS", "-"}:
                return True
    return False


def _calculate_pursuit_time(result_time: str, start_delay: str) -> str:
    """Return pursuit time excluding start delay."""
    result_secs = parse_time_seconds(result_time)
    if result_secs is None:
        return "-"
    delay_secs = parse_time_seconds(start_delay)
    if delay_secs is None:
        delay_secs = 0
    pursuit_secs = result_secs - delay_secs
    if pursuit_secs < 0:
        return "-"
    return format_seconds(pursuit_secs)


def _fetch_analytic_map(race_id: str, type_id: str) -> dict[str, str]:
    """Fetch analytic times keyed by IBUId/Bib/Name."""
    try:
        analytic = get_analytic_results(race_id, type_id)
    except BiathlonError:
        return {}
    times: dict[str, str] = {}
    for res in analytic.get("Results", []):
        if res.get("IsTeam"):
            continue
        ident = res.get("IBUId") or res.get("Bib") or res.get("Name")
        if not ident:
            continue
        times[str(ident)] = get_first_time(res, ["TotalTime", "Result"]) or "-"
    return times


def _fetch_stage_times(race_id: str, prefix: str, suffix: str, count: int) -> dict[int, dict[str, str]]:
    """Fetch analytic stage times in parallel."""
    from concurrent.futures import ThreadPoolExecutor

    def fetch_one(idx: int) -> tuple[int, dict[str, str]]:
        type_id = f"{prefix}{idx}{suffix}"
        return idx, _fetch_analytic_map(race_id, type_id)

    stages: dict[int, dict[str, str]] = {}
    with ThreadPoolExecutor(max_workers=count) as executor:
        for idx, times in executor.map(fetch_one, range(1, count + 1)):
            stages[idx] = times
    return stages


def _lookup_analytic_time(times: dict[str, str], res: dict) -> str:
    """Return analytic time for a result using multiple identifier keys."""
    for key in (res.get("IBUId"), res.get("Bib"), res.get("Name"), res.get("ShortName")):
        if key and str(key) in times:
            return times[str(key)]
    return ""


def _parse_shootings(value: str | None) -> list[int]:
    """Parse shootings string like '0+1+0+2' into list of ints."""
    if not value:
        return []
    parts = [p.strip() for p in str(value).split("+") if p.strip()]
    misses: list[int] = []
    for part in parts:
        try:
            misses.append(int(part))
        except ValueError:
            misses.append(0)
    return misses


def _shooting_totals(misses: list[int]) -> tuple[int, int, int]:
    """Return (misses_total, prone_total, standing_total)."""
    misses_total = sum(misses) if misses else 0
    prone = 0
    standing = 0
    if len(misses) >= 4:
        prone = misses[0] + misses[1]
        standing = misses[2] + misses[3]
    elif len(misses) == 2:
        prone = misses[0]
        standing = misses[1]
    elif len(misses) == 1:
        prone = misses[0]
    return misses_total, prone, standing


def _shooting_stages(misses: list[int], stages: int) -> tuple[str, str, str, str]:
    """Return stage misses for Prone1/Prone2/Standing1/Standing2."""
    stage_vals = ["-", "-", "-", "-"]
    if not misses or stages <= 0:
        return stage_vals[0], stage_vals[1], stage_vals[2], stage_vals[3]
    if stages <= 2:
        stage_vals[0] = str(misses[0])
        if len(misses) >= 2:
            stage_vals[2] = str(misses[1])
        return stage_vals[0], stage_vals[1], stage_vals[2], stage_vals[3]
    for idx, val in enumerate(misses[:4]):
        stage_vals[idx] = str(val)
    return stage_vals[0], stage_vals[1], stage_vals[2], stage_vals[3]


def _find_latest_race_with_results_any() -> tuple[str, dict]:
    """Return the most recent race id with completed results (incl. relay)."""
    now = datetime.datetime.now(datetime.timezone.utc)
    season_id = get_current_season_id()
    events = get_events(season_id, level=1)

    races: list[tuple[str, str, str]] = []
    for event in events:
        event_id = event.get("EventId")
        if not event_id:
            continue
        for race in get_races(event_id):
            start_key = get_race_start_key(race)
            race_id = race.get("RaceId") or race.get("Id") or ""
            discipline = str(race.get("DisciplineId") or "").upper()
            if race_id:
                races.append((start_key, race_id, discipline))

    races.sort(reverse=True)

    for start_key, race_id, discipline in races:
        try:
            payload = get_race_results(race_id)
        except BiathlonError:
            continue
        comp = payload.get("Competition") or {}
        start_raw = comp.get("StartTime") or start_key
        start_dt = parse_start_datetime(start_raw if isinstance(start_raw, str) else None)
        if start_dt and start_dt > now:
            continue
        if discipline in (RELAY_DISCIPLINE, SINGLE_MIXED_RELAY_DISCIPLINE):
            if _has_completed_relay_results(payload):
                return race_id, payload
        else:
            if _has_completed_results(payload):
                return race_id, payload

    raise BiathlonError("No completed races with results found")


def _find_latest_race_by_discipline(
    discipline: str, mixed_mode: str, cat_filter: str | None
) -> tuple[str, dict]:
    """Return the most recent race id with completed results for a discipline.

    mixed_mode: "any", "mixed-only", or "non-mixed".
    """
    now = datetime.datetime.now(datetime.timezone.utc)
    season_id = get_current_season_id()
    events = get_events(season_id, level=1)

    races: list[tuple[str, str, str, str]] = []
    for event in events:
        event_id = event.get("EventId")
        if not event_id:
            continue
        for race in get_races(event_id):
            start_key = get_race_start_key(race)
            race_id = race.get("RaceId") or race.get("Id") or ""
            race_disc = str(race.get("DisciplineId") or "").upper()
            race_cat = str(race.get("catId") or race.get("CatId") or "").upper()
            if race_id:
                races.append((start_key, race_id, race_disc, race_cat))

    races.sort(reverse=True)

    for start_key, race_id, race_disc, race_cat in races:
        if race_disc != discipline:
            continue
        try:
            payload = get_race_results(race_id)
        except BiathlonError:
            continue
        comp = payload.get("Competition") or {}
        start_raw = comp.get("StartTime") or start_key
        start_dt = parse_start_datetime(start_raw if isinstance(start_raw, str) else None)
        if start_dt and start_dt > now:
            continue
        comp_cat = str(comp.get("catId") or comp.get("CatId") or race_cat or "").upper()
        if mixed_mode == "mixed-only" and comp_cat and comp_cat != RELAY_MIXED_CAT:
            continue
        if mixed_mode == "non-mixed" and comp_cat == RELAY_MIXED_CAT:
            continue
        if cat_filter and comp_cat and comp_cat != cat_filter:
            continue

        if discipline in (RELAY_DISCIPLINE, SINGLE_MIXED_RELAY_DISCIPLINE):
            if _has_completed_relay_results(payload):
                return race_id, payload
        else:
            if _has_completed_results(payload):
                return race_id, payload

    raise BiathlonError(f"No completed races with results found for discipline {discipline}")


def _get_discipline(payload: dict) -> str:
    """Return the discipline id for a race payload."""
    comp = payload.get("Competition") or {}
    return str(comp.get("DisciplineId") or "").upper()


def _sort_rows(rows: list[dict], key: str, relay: bool) -> list[dict]:
    """Sort rows by a given key."""
    def time_key(value: object) -> tuple:
        sec = parse_time_seconds(str(value)) if value not in ("", None, "-") else None
        if sec is None:
            return (1, float("inf"))
        return (0, sec)

    def int_key(value: object) -> tuple:
        try:
            return (0, int(value))
        except (TypeError, ValueError):
            return (1, 10**9)

    def shooting_key(value: object) -> tuple:
        shooting = parse_relay_shooting(value) if value not in ("", None, "-") else None
        if shooting:
            return (0, shooting[0], shooting[1])
        return (1, 9999, 9999)

    def row_key(row: dict) -> tuple:
        dns_flag = row.get("dns", False)
        value = row.get(key, "")
        if key in {"rank", "startrank", "misses"} and not relay:
            return (1, 0, 0) if dns_flag else (0, *int_key(value))
        if key == "gain" and not relay:
            if dns_flag:
                return (1, 0, 0)
            try:
                return (0, -int(value))
            except (TypeError, ValueError):
                return (0, 10**9)
        if relay and key in {"misses"}:
            return (1, 0, 0) if dns_flag else (0, *shooting_key(value))
        return (1, 0, 0) if dns_flag else (0, *time_key(value))

    return sorted(rows, key=row_key)


def _build_relay_rows(payload: dict, race_id: str, first_n: int, discipline: str) -> list[dict]:
    """Build relay team rows with leg details."""
    from .relay import _fetch_analytic_times, _fetch_leg_lap_times

    results = payload.get("Results", [])
    if not results:
        return []

    crst_times = _fetch_analytic_times(race_id, "CRST")
    course_laps = _fetch_leg_lap_times(race_id, "CRS", "", 12, 3)
    range_laps = _fetch_leg_lap_times(race_id, "RNG", "", 8, 2)
    shooting_laps = _fetch_leg_lap_times(race_id, "S", "TM", 8, 2)
    rngt_times = _fetch_analytic_times(race_id, "RNGT")
    sttm_times = _fetch_analytic_times(race_id, "STTM")

    team_results = [r for r in results if r.get("IsTeam")]
    leg_results = [r for r in results if not r.get("IsTeam")]

    def team_sort_key(r: dict) -> tuple:
        rank = r.get("Rank") or r.get("ResultOrder") or 9999
        try:
            return (0, int(rank))
        except (TypeError, ValueError):
            return (1, str(rank))

    team_results.sort(key=team_sort_key)
    if first_n > 0:
        team_results = team_results[:first_n]

    legs_by_bib: dict[str, list[dict]] = {}
    for leg in leg_results:
        bib = str(leg.get("Bib") or "")
        if bib:
            legs_by_bib.setdefault(bib, []).append(leg)
    for bib in legs_by_bib:
        legs_by_bib[bib].sort(key=lambda x: x.get("Leg", 0))

    num_legs = 4
    is_single_mixed = discipline == SINGLE_MIXED_RELAY_DISCIPLINE
    rows = []
    for team in team_results:
        bib = str(team.get("Bib") or "")
        team_rank = team.get("Rank") or team.get("ResultOrder") or ""
        team_name = team.get("Name") or team.get("ShortName") or ""
        nat = team.get("Nat") or ""
        total_time = team.get("TotalTime") or team.get("Result") or "-"
        behind = team.get("Behind") or ""

        result_secs = parse_time_seconds(str(total_time)) if total_time != "-" else None
        team_course_secs = 0.0
        team_range_secs = 0.0
        team_shooting_secs = 0.0
        has_course = False
        has_range = False
        has_shooting = False

        team_legs = legs_by_bib.get(bib, [])
        leg_prone: list[tuple[int, int] | None] = []
        leg_standing: list[tuple[int, int] | None] = []
        leg_names: list[str] = []
        leg_results_list: list[str] = []
        leg_total_times: list[str] = []
        leg_behinds: list[str] = []
        leg_misses: list[str] = []
        leg_prone_strs: list[str] = []
        leg_standing_strs: list[str] = []
        leg_course_laps: list[list[str]] = []
        leg_range_laps: list[list[str]] = []
        leg_shooting_laps: list[list[str]] = []
        leg_course_totals: list[str] = []

        for i in range(1, num_legs + 1):
            leg_key = (bib, i)
            if leg_key in crst_times:
                team_course_secs += crst_times[leg_key]
                has_course = True
            if leg_key in rngt_times:
                team_range_secs += rngt_times[leg_key]
                has_range = True
            if leg_key in sttm_times:
                team_shooting_secs += sttm_times[leg_key]
                has_shooting = True

            leg_data = next((lg for lg in team_legs if lg.get("Leg") == i), None)
            lap_values = []
            leg_lap_times = {}
            if leg_data:
                lookup_keys = []
                for ident in (leg_data.get("IBUId"), leg_data.get("Bib"), bib, leg_data.get("Name")):
                    if ident:
                        lookup_keys.append((str(ident), i))
                        if is_single_mixed and i > 2:
                            lookup_keys.append((str(ident), i - 2))
                for key in lookup_keys:
                    if key in course_laps:
                        leg_lap_times = course_laps[key]
                        break
            if not leg_lap_times:
                leg_lap_times = course_laps.get(leg_key, {})
                if is_single_mixed and i > 2 and not leg_lap_times:
                    leg_lap_times = course_laps.get((bib, i - 2), {})
            for lap_idx in range(1, 4):
                lap_values.append(leg_lap_times.get(f"lap{lap_idx}", "-"))
            leg_course_laps.append(lap_values)

            range_values = []
            range_times = {}
            if leg_data:
                lookup_keys = []
                for ident in (leg_data.get("IBUId"), leg_data.get("Bib"), bib, leg_data.get("Name")):
                    if ident:
                        lookup_keys.append((str(ident), i))
                        if is_single_mixed and i > 2:
                            lookup_keys.append((str(ident), i - 2))
                for key in lookup_keys:
                    if key in range_laps:
                        range_times = range_laps[key]
                        break
            if not range_times:
                range_times = range_laps.get(leg_key, {})
                if is_single_mixed and i > 2 and not range_times:
                    range_times = range_laps.get((bib, i - 2), {})
            for lap_idx in range(1, 3):
                range_values.append(range_times.get(f"lap{lap_idx}", "-"))
            leg_range_laps.append(range_values)

            shooting_values = []
            shooting_times = {}
            if leg_data:
                lookup_keys = []
                for ident in (leg_data.get("IBUId"), leg_data.get("Bib"), bib, leg_data.get("Name")):
                    if ident:
                        lookup_keys.append((str(ident), i))
                        if is_single_mixed and i > 2:
                            lookup_keys.append((str(ident), i - 2))
                for key in lookup_keys:
                    if key in shooting_laps:
                        shooting_times = shooting_laps[key]
                        break
            if not shooting_times:
                shooting_times = shooting_laps.get(leg_key, {})
                if is_single_mixed and i > 2 and not shooting_times:
                    shooting_times = shooting_laps.get((bib, i - 2), {})
            for lap_idx in range(1, 3):
                shooting_values.append(shooting_times.get(f"lap{lap_idx}", "-"))
            leg_shooting_laps.append(shooting_values)

            if leg_data:
                shootings = parse_relay_shootings(leg_data.get("Shootings"))
                if shootings:
                    prone, standing = shootings
                    leg_prone.append(prone)
                    leg_standing.append(standing)
                else:
                    leg_prone.append(None)
                    leg_standing.append(None)

                leg_names.append(leg_data.get("ShortName") or leg_data.get("Name") or "-")
                leg_result = get_first_time(leg_data, ["TotalTime", "Result"]) or "-"
                leg_results_list.append(leg_result)
                leg_total_times.append(leg_result)
                leg_behinds.append(leg_data.get("Behind") or "-")
                if shootings:
                    leg_prone_strs.append(format_relay_shooting(*shootings[0]))
                    leg_standing_strs.append(format_relay_shooting(*shootings[1]))
                    total_p = shootings[0][0] + shootings[1][0]
                    total_s = shootings[0][1] + shootings[1][1]
                    leg_misses.append(format_relay_shooting(total_p, total_s))
                else:
                    leg_prone_strs.append("-")
                    leg_standing_strs.append("-")
                    total = parse_relay_shooting(leg_data.get("ShootingTotal"))
                    leg_misses.append(format_relay_shooting(*total) if total else "-")
            else:
                leg_prone.append(None)
                leg_standing.append(None)
                leg_names.append("-")
                leg_results_list.append("-")
                leg_total_times.append("-")
                leg_behinds.append("-")
                leg_misses.append("-")
                leg_prone_strs.append("-")
                leg_standing_strs.append("-")
                if not lap_values:
                    leg_course_laps.append(["-", "-", "-"])

            leg_course = "-"
            if leg_key in crst_times:
                leg_course = format_seconds(crst_times[leg_key])
            leg_course_totals.append(leg_course)

        leg_times: list[str] = []
        prev_secs: float | None = None
        for i in range(num_legs):
            curr_text = leg_total_times[i]
            curr_secs = parse_time_seconds(curr_text) if curr_text not in ("", None, "-") else None
            if i == 0:
                leg_times.append(format_seconds(curr_secs) if curr_secs is not None else curr_text)
            else:
                if curr_secs is not None and prev_secs is not None:
                    leg_times.append(format_seconds(curr_secs - prev_secs))
                else:
                    leg_times.append("-")
            if curr_secs is not None:
                prev_secs = curr_secs

        course_str = format_seconds(team_course_secs) if has_course else "-"
        range_str = format_seconds(team_range_secs) if has_range else "-"
        shooting_str = format_seconds(team_shooting_secs) if has_shooting else "-"

        penalty_str = "-"
        if result_secs is not None and has_course and has_range:
            penalty_secs = result_secs - team_course_secs - team_range_secs
            if penalty_secs >= 0:
                penalty_str = format_seconds(penalty_secs)

        total_prone = add_relay_shootings(leg_prone)
        total_standing = add_relay_shootings(leg_standing)
        total_misses = (total_prone[0] + total_standing[0], total_prone[1] + total_standing[1])

        prone_str = format_relay_shooting(*total_prone)
        standing_str = format_relay_shooting(*total_standing)
        misses_str = format_relay_shooting(*total_misses)

        row = {
            "rank": team_rank,
            "team": team_name,
            "nat": nat,
            "result": total_time,
            "behind": behind,
            "course": course_str,
            "range": range_str,
            "shooting": shooting_str,
            "penalty": penalty_str,
            "misses": misses_str,
            "dns": is_dns(team),
        }
        for i in range(1, num_legs + 1):
            row[f"leg{i}_name"] = leg_names[i - 1]
            row[f"leg{i}_result"] = leg_results_list[i - 1]
            row[f"leg{i}_behind"] = leg_behinds[i - 1]
            row[f"leg{i}_course"] = leg_course_totals[i - 1]
            row[f"leg{i}_miss"] = leg_misses[i - 1]
            row[f"leg{i}_prone"] = leg_prone_strs[i - 1]
            row[f"leg{i}_standing"] = leg_standing_strs[i - 1]
            row[f"leg{i}_courselap1"] = leg_course_laps[i - 1][0]
            row[f"leg{i}_courselap2"] = leg_course_laps[i - 1][1]
            row[f"leg{i}_courselap3"] = leg_course_laps[i - 1][2]
            row[f"leg{i}_range1"] = leg_range_laps[i - 1][0]
            row[f"leg{i}_range2"] = leg_range_laps[i - 1][1]
            row[f"leg{i}_shooting1"] = leg_shooting_laps[i - 1][0]
            row[f"leg{i}_shooting2"] = leg_shooting_laps[i - 1][1]
            row[f"leg{i}_time"] = leg_times[i - 1]
        rows.append(row)

    return rows


def handle_results(args: argparse.Namespace) -> int:
    """List results for a race (unified output)."""
    if getattr(args, "race", "") and getattr(args, "discipline", ""):
        print("error: --race and --discipline cannot be used together", file=sys.stderr)
        return 1
    if getattr(args, "race", "") and getattr(args, "men", False):
        print("error: --race and --men cannot be used together", file=sys.stderr)
        return 1

    if getattr(args, "race", ""):
        race_id = args.race
        payload = get_race_results(race_id)
    elif getattr(args, "discipline", ""):
        disc_arg = args.discipline
        is_men = getattr(args, "men", False)
        disc_map = {
            "individual": ("IN", "any"),
            "sprint": ("SP", "any"),
            "pursuit": ("PU", "any"),
            "mass-start": ("MS", "any"),
            "relay": (RELAY_DISCIPLINE, "non-mixed"),
            "mixed-relay": (RELAY_DISCIPLINE, "mixed-only"),
            "single-mixed-relay": (SINGLE_MIXED_RELAY_DISCIPLINE, "mixed-only"),
        }
        disc_code, mixed_mode = disc_map.get(disc_arg, ("", "any"))
        if not disc_code:
            print("error: unknown discipline", file=sys.stderr)
            return 1
        if disc_arg in {"mixed-relay", "single-mixed-relay"} and is_men:
            print("error: --men is not supported for mixed relay races", file=sys.stderr)
            return 1
        cat_filter = RELAY_MEN_CAT if is_men else RELAY_WOMEN_CAT
        if disc_arg in {"mixed-relay", "single-mixed-relay"}:
            cat_filter = None
        race_id, payload = _find_latest_race_by_discipline(disc_code, mixed_mode, cat_filter)
    else:
        race_id, payload = _find_latest_race_with_results_any()

    discipline = _get_discipline(payload)
    show_detail = getattr(args, "detail", False)

    if discipline in (RELAY_DISCIPLINE, SINGLE_MIXED_RELAY_DISCIPLINE):
        rows = _build_relay_rows(payload, race_id, getattr(args, "first", 0), discipline)
        if not rows:
            print(f"no results found for race {race_id}", file=sys.stderr)
            return 1
        if getattr(args, "country", ""):
            country_filter = args.country.upper()
            rows = [r for r in rows if r.get("nat", "").upper() == country_filter]

        sort_col = getattr(args, "sort", "").lower().replace(" ", "")
        show_sort_rank = False
        sort_header = ""
        detail_sort_key = ""
        if sort_col:
            sort_map = {
                "results": "result",
                "result": "result",
                "behind": "behind",
                "course": "course",
                "range": "range",
                "shooting": "shooting",
                "shoot": "shooting",
                "penalty": "penalty",
                "misses": "misses",
                "miss": "misses",
            }
            detail_sort_map = {
                "leg": "leg",
                "biathlete": "biathlete",
                "legresult": "leg_result",
                "legbehind": "leg_behind",
                "legcourse": "leg_course",
                "lap1": "lap1",
                "lap2": "lap2",
                "lap3": "lap3",
                "r1": "range1",
                "r2": "range2",
                "s1": "shoot1",
                "s2": "shoot2",
                "prone": "leg_prone",
                "standing": "leg_standing",
                "miss": "leg_miss",
                "legpenalty": "leg_penalty",
            }
            sort_key = sort_map.get(sort_col)
            detail_sort_key = detail_sort_map.get(sort_col, "")
            if not sort_key and not detail_sort_key:
                valid = ", ".join(list(sort_map.keys()) + list(detail_sort_map.keys()))
                print(f"error: sort must be one of {valid}", file=sys.stderr)
                return 1
            if sort_key:
                rows = _sort_rows(rows, sort_key, relay=True)
                show_sort_rank = True
                sort_header = {
                    "result": "Results",
                    "behind": "Behind",
                    "course": "Course",
                    "range": "Range",
                    "shooting": "Shoot",
                    "penalty": "Penalty",
                    "misses": "Miss",
                }.get(sort_key, "")
            elif detail_sort_key:
                show_sort_rank = True
                sort_header = {
                    "leg": "Leg",
                    "biathlete": "Biathlete",
                    "leg_result": "LegResult",
                    "leg_behind": "LegBehind",
                    "leg_course": "LegCourse",
                    "lap1": "Lap1",
                    "lap2": "Lap2",
                    "lap3": "Lap3",
                    "range1": "R1",
                    "range2": "R2",
                    "shoot1": "S1",
                    "shoot2": "S2",
                    "leg_prone": "Prone",
                    "leg_standing": "Stand",
                    "leg_miss": "Miss",
                    "leg_penalty": "LegPenalty",
                }.get(detail_sort_key, "")

        limit_n = getattr(args, "limit", 25)
        if limit_n > 0:
            rows = rows[:limit_n]

        print(format_race_header(payload, race_id))
        if show_detail:
            headers = [
                "Rank", "Team", "Results", "Behind", "Course", "Range", "Shoot", "Penalty",
                "Miss",
                "Leg", "Biathlete", "LegResult", "LegBehind", "LegCourse",
                "Lap1", "Lap2", "Lap3",
                "R1", "R2", "S1", "S2",
                "Prone", "Stand",
                "Miss", "LegPenalty",
            ]
        else:
            headers = [
                "Rank", "Team", "Results", "Behind",
                "Course", "Range", "Shoot", "Penalty", "Miss",
            ]
        if show_sort_rank:
            headers.insert(0, "Sort")

        render_rows: list[list[str]] = []
        row_styles: list[str] = []
        if show_detail:
            detail_rows: list[dict] = []
            for row in rows:
                for leg in range(1, 5):
                    detail_rows.append(
                        {
                            "rank": row["rank"],
                            "team": row["team"],
                            "result": row["result"],
                            "behind": row["behind"],
                            "course": row["course"],
                            "range": row["range"],
                            "shooting": row["shooting"],
                            "penalty": row["penalty"],
                            "misses": row["misses"],
                            "leg": leg,
                            "biathlete": row[f"leg{leg}_name"],
                            "leg_result": row[f"leg{leg}_result"],
                            "leg_behind": row[f"leg{leg}_behind"],
                            "leg_course": row[f"leg{leg}_course"],
                            "lap1": row[f"leg{leg}_courselap1"],
                            "lap2": row[f"leg{leg}_courselap2"],
                            "lap3": row[f"leg{leg}_courselap3"],
                            "range1": row[f"leg{leg}_range1"],
                            "range2": row[f"leg{leg}_range2"],
                            "shoot1": row[f"leg{leg}_shooting1"],
                            "shoot2": row[f"leg{leg}_shooting2"],
                            "leg_prone": row[f"leg{leg}_prone"],
                            "leg_standing": row[f"leg{leg}_standing"],
                            "leg_miss": row[f"leg{leg}_miss"],
                            "leg_time": row[f"leg{leg}_time"],
                            "dns": row.get("dns", False),
                        }
                    )
            def detail_sort(entry: dict) -> tuple:
                dns_flag = entry.get("dns", False)
                val = entry.get(detail_sort_key)
                if detail_sort_key in {"leg"}:
                    try:
                        return (1, 0) if dns_flag else (0, int(val))
                    except (TypeError, ValueError):
                        return (0, 10**9)
                if detail_sort_key in {"leg_miss", "leg_prone", "leg_standing"}:
                    shooting = parse_relay_shooting(val) if val not in ("", None, "-") else None
                    if shooting:
                        return (1, 0, 0) if dns_flag else (0, shooting[0], shooting[1])
                    return (0, 9999, 9999)
                if detail_sort_key in {"leg_result", "leg_behind", "leg_time", "leg_course", "lap1", "lap2", "lap3", "range1", "range2", "shoot1", "shoot2", "leg_penalty"}:
                    sec = parse_time_seconds(str(val)) if val not in ("", None, "-") else None
                    if sec is None:
                        return (1, float("inf"))
                    return (1, float("inf")) if dns_flag else (0, sec)
                text = str(val or "").strip().lower()
                return (text == "", text)

            if detail_sort_key:
                detail_rows = sorted(detail_rows, key=detail_sort)

            for entry in detail_rows:
                leg_time_secs = parse_time_seconds(entry.get("leg_time"))
                leg_course_secs = parse_time_seconds(entry.get("leg_course"))
                range1_secs = parse_time_seconds(entry.get("range1"))
                range2_secs = parse_time_seconds(entry.get("range2"))
                if None in (leg_time_secs, leg_course_secs, range1_secs, range2_secs):
                    entry["leg_penalty"] = "-"
                else:
                    penalty_secs = leg_time_secs - leg_course_secs - range1_secs - range2_secs
                    entry["leg_penalty"] = format_seconds(penalty_secs) if penalty_secs >= 0 else "-"

            for idx, entry in enumerate(detail_rows, start=1):
                render_rows.append(
                    [
                        entry["rank"],
                        entry["team"],
                        entry["result"],
                        entry["behind"],
                        entry["course"],
                        entry["range"],
                        entry["shooting"],
                        entry["penalty"],
                        entry["misses"],
                        entry["leg"],
                        entry["biathlete"],
                        entry["leg_result"],
                        entry["leg_behind"],
                        entry["leg_course"],
                        entry["lap1"],
                        entry["lap2"],
                        entry["lap3"],
                        entry["range1"],
                        entry["range2"],
                        entry["shoot1"],
                        entry["shoot2"],
                        entry["leg_prone"],
                        entry["leg_standing"],
                        entry["leg_miss"],
                        entry["leg_penalty"],
                    ]
                )
                if show_sort_rank:
                    render_rows[-1].insert(0, idx)
                row_styles.append(rank_style(entry.get("rank")))
        else:
            for idx, row in enumerate(rows, start=1):
                render_rows.append(
                    [
                        row["rank"],
                        row["team"],
                        row["result"],
                        row["behind"],
                        row["course"],
                        row["range"],
                        row["shooting"],
                        row["penalty"],
                        row["misses"],
                    ]
                )
                if show_sort_rank:
                    render_rows[-1].insert(0, idx)
                row_styles.append(rank_style(row.get("rank")))
        highlight_headers = None
        if show_sort_rank and sort_header and sort_header in headers:
            highlight_headers = [headers.index(sort_header)]
        render_table(
            headers,
            render_rows,
            pretty=is_pretty_output(args),
            row_styles=row_styles if is_pretty_output(args) else None,
            highlight_headers=highlight_headers,
        )
        return 0

    results = extract_results(payload)
    if not results:
        print(f"no results found for race {race_id}", file=sys.stderr)
        return 1

    top_n = getattr(args, "top", 0)
    if top_n > 0:
        cat_id = (payload.get("Competition") or {}).get("catId", "").upper()
        if cat_id in ("SM", "SW"):
            top_ibu_ids = _get_top_n_ibu_ids(cat_id, top_n)
            if top_ibu_ids:
                results = [r for r in results if r.get("IBUId") in top_ibu_ids]

    first_n = getattr(args, "first", 0)
    if first_n > 0:
        results = results[:first_n]

    base_secs = base_time_seconds(results)
    laps = SKI_LAPS.get(discipline, 0)
    stages = SHOOTING_STAGES.get(discipline, 0)

    # Fetch all analytic data in parallel
    from concurrent.futures import ThreadPoolExecutor, as_completed

    course_times: dict[str, str] = {}
    ski_times: dict[str, str] = {}
    range_times: dict[str, str] = {}
    shooting_times: dict[str, str] = {}
    course_laps: dict[int, dict[str, str]] = {}
    range_laps: dict[int, dict[str, str]] = {}
    shooting_laps: dict[int, dict[str, str]] = {}

    # Build list of all fetches needed
    fetches: list[tuple[str, str, int | None]] = [
        ("map", "CRST", None),
        ("map", "SKIT", None),
        ("map", "RNGT", None),
        ("map", "STTM", None),
    ]
    if show_detail and laps:
        for i in range(1, laps + 1):
            fetches.append(("course_lap", f"CRS{i}", i))
    if show_detail and stages:
        for i in range(1, stages + 1):
            fetches.append(("range_lap", f"RNG{i}", i))
            fetches.append(("shooting_lap", f"S{i}TM", i))

    def fetch_one(fetch_type: str, type_id: str, idx: int | None) -> tuple[str, str, int | None, dict[str, str]]:
        return fetch_type, type_id, idx, _fetch_analytic_map(race_id, type_id)

    with ThreadPoolExecutor(max_workers=len(fetches)) as executor:
        futures = [executor.submit(fetch_one, ft, tid, idx) for ft, tid, idx in fetches]
        for future in as_completed(futures):
            fetch_type, type_id, idx, data = future.result()
            if fetch_type == "map":
                if type_id == "CRST":
                    course_times = data
                elif type_id == "SKIT":
                    ski_times = data
                elif type_id == "RNGT":
                    range_times = data
                elif type_id == "STTM":
                    shooting_times = data
            elif fetch_type == "course_lap" and idx is not None:
                course_laps[idx] = data
            elif fetch_type == "range_lap" and idx is not None:
                range_laps[idx] = data
            elif fetch_type == "shooting_lap" and idx is not None:
                shooting_laps[idx] = data

    rows = []
    for res in results:
        ident = str(res.get("IBUId") or res.get("Bib") or res.get("Name") or "")
        rank = res.get("Rank") or res.get("ResultOrder") or ""
        name = res.get("Name") or res.get("ShortName") or ""
        nat = res.get("Nat") or ""
        result_time = normalize_result_time(res, base_secs)
        course_time = _lookup_analytic_time(course_times, res) or get_first_time(
            res, ["TotalCourseTime", "CourseTime", "RunTime"]
        ) or "-"
        range_time = _lookup_analytic_time(range_times, res) or get_first_time(
            res, ["TotalRangeTime", "RangeTime"]
        ) or "-"
        shooting_time = _lookup_analytic_time(shooting_times, res) or get_first_time(
            res, ["TotalShootingTime", "ShootingTime"]
        ) or "-"
        ski_time = _lookup_analytic_time(ski_times, res) or get_first_time(res, [
            "TotalSkiTime", "SkiTime", "SkiTimeTotal", "SKITime", "Ski",
        ]) or "-"

        misses_list = _parse_shootings(res.get("Shootings") or res.get("ShootingTotal"))
        misses_total, prone_total, standing_total = _shooting_totals(misses_list)
        prone1, prone2, standing1, standing2 = _shooting_stages(misses_list, stages)

        start_rank = res.get("StartOrder") or res.get("StartPosition") or "-"
        start_delay = res.get("StartInfo") or "-"
        gain = "-"
        try:
            gain = int(start_rank) - int(rank)
        except (TypeError, ValueError):
            gain = "-"
        pursuit_time = "-"
        if discipline == "PU":
            pursuit_time = _calculate_pursuit_time(result_time, str(start_delay))

        if discipline == "IN":
            penalty = format_seconds(misses_total * 60) if misses_list else "-"
        elif discipline == "SI":
            penalty = format_seconds(misses_total * 45) if misses_list else "-"
        else:
            base_time = pursuit_time if discipline == "PU" else result_time
            result_secs = parse_time_seconds(base_time)
            course_secs = parse_time_seconds(course_time)
            range_secs = parse_time_seconds(range_time)
            if result_secs is None or course_secs is None or range_secs is None:
                penalty = "-"
            else:
                penalty_secs = result_secs - course_secs - range_secs
                penalty = format_seconds(penalty_secs) if penalty_secs >= 0 else "-"

        row = {
            "rank": rank,
            "biathlete": name,
            "country": nat,
            "startrank": start_rank,
            "gain": gain,
            "startdelay": start_delay,
            "result": result_time,
            "pursuittime": pursuit_time,
            "ski": ski_time,
            "course": course_time,
            "range": range_time,
            "shooting": shooting_time,
            "penalty": penalty,
            "misses": str(misses_total) if misses_list else "-",
            "prone1": prone1,
            "prone2": prone2,
            "standing1": standing1,
            "standing2": standing2,
            "dns": is_dns(res),
            "ibu_id": res.get("IBUId") or "",
        }
        if show_detail:
            for lap in range(1, laps + 1):
                row[f"courselap{lap}"] = _lookup_analytic_time(course_laps.get(lap, {}), res) or "-"
            for stage in range(1, stages + 1):
                row[f"range{stage}"] = _lookup_analytic_time(range_laps.get(stage, {}), res) or "-"
                row[f"shoot{stage}"] = _lookup_analytic_time(shooting_laps.get(stage, {}), res) or "-"
        rows.append(row)

    country_filter = getattr(args, "country", "").upper()
    if country_filter:
        rows = [r for r in rows if r.get("country", "").upper() == country_filter]

    sort_col = getattr(args, "sort", "").lower().replace(" ", "")
    show_sort_rank = False
    sort_header = ""
    if sort_col:
        sort_map = {
            "results": "result",
            "result": "result",
            "pursuittime": "pursuittime",
            "pursuit": "pursuittime",
            "startrank": "startrank",
            "start": "startrank",
            "startdelay": "startdelay",
            "delay": "startdelay",
            "gain": "gain",
            "ski": "ski",
            "course": "course",
            "range": "range",
            "shooting": "shooting",
            "shoot": "shooting",
            "penalty": "penalty",
            "misses": "misses",
            "miss": "misses",
        }
        if show_detail:
            detail_map = {
                "lap1": "courselap1",
                "lap2": "courselap2",
                "lap3": "courselap3",
                "lap4": "courselap4",
                "lap5": "courselap5",
                "r1": "range1",
                "r2": "range2",
                "r3": "range3",
                "r4": "range4",
                "s1": "shoot1",
                "s2": "shoot2",
                "s3": "shoot3",
                "s4": "shoot4",
                "pr1": "prone1",
                "pr2": "prone2",
                "st1": "standing1",
                "st2": "standing2",
            }
            if discipline == "SP":
                detail_map.update({"pr": "prone1", "st": "standing1"})
            sort_map.update(detail_map)
        sort_key = sort_map.get(sort_col)
        if not sort_key:
            valid = ", ".join(sort_map.keys())
            print(f"error: sort must be one of {valid}", file=sys.stderr)
            return 1
        rows = _sort_rows(rows, sort_key, relay=False)
        show_sort_rank = True
        sort_header = {
            "result": "Results",
            "pursuittime": "Pursuit",
            "startrank": "Start",
            "startdelay": "Delay",
            "gain": "Gain",
            "ski": "Ski",
            "course": "Course",
            "range": "Range",
            "shooting": "Shoot",
            "penalty": "Penalty",
            "misses": "Miss",
            "courselap1": "Lap1",
            "courselap2": "Lap2",
            "courselap3": "Lap3",
            "courselap4": "Lap4",
            "courselap5": "Lap5",
            "range1": "R1",
            "range2": "R2",
            "range3": "R3",
            "range4": "R4",
            "shoot1": "S1",
            "shoot2": "S2",
            "shoot3": "S3",
            "shoot4": "S4",
            "prone1": "Pr1",
            "prone2": "Pr2",
            "standing1": "St1",
            "standing2": "St2",
        }.get(sort_key, "")

    limit_n = getattr(args, "limit", 25)
    if limit_n > 0:
        rows = rows[:limit_n]

    print(format_race_header(payload, race_id))

    headers: list[str] = []
    if discipline == "PU":
        headers = [
            "Rank", "Biathlete", "Nat", "Start", "Gain", "Delay",
            "Results", "Pursuit", "Course", "Range", "Shoot", "Penalty",
            "Miss",
        ]
    else:
        headers = [
            "Rank", "Biathlete", "Nat", "Results",
            "Ski" if discipline in {"IN", "SI"} else "",
            "Course", "Range", "Shoot", "Penalty", "Miss",
        ]
        headers = [h for h in headers if h]

    if show_detail:
        if discipline == "SP":
            detail_headers = [
                "Lap1", "Lap2", "Lap3",
                "R1", "R2", "S1", "S2", "Pr", "St",
            ]
        else:
            detail_headers = [
                "Lap1", "Lap2", "Lap3",
            ]
            if laps >= 4:
                detail_headers.append("Lap4")
            if laps >= 5:
                detail_headers.append("Lap5")
            detail_headers.extend([
                "R1", "R2", "R3", "R4",
                "S1", "S2", "S3", "S4",
                "Pr1", "Pr2", "St1", "St2",
            ])
        headers.extend(detail_headers)
    if show_sort_rank:
        headers.insert(0, "Sort")

    render_rows: list[list[str]] = []
    for idx, row in enumerate(rows, start=1):
        base = []
        if discipline == "PU":
            gain_val = row["gain"]
            if isinstance(gain_val, int) and gain_val > 0:
                gain_val = f"+{gain_val}"
            base = [
                row["rank"],
                row["biathlete"],
                row["country"],
                row["startrank"],
                gain_val,
                row["startdelay"],
                row["result"],
                row["pursuittime"],
                row["course"],
                row["range"],
                row["shooting"],
                row["penalty"],
                row["misses"],
            ]
        else:
            base = [
                row["rank"],
                row["biathlete"],
                row["country"],
                row["result"],
            ]
            if discipline in {"IN", "SI"}:
                base.append(row["ski"])
            base.extend([
                row["course"],
                row["range"],
                row["shooting"],
                row["penalty"],
                row["misses"],
            ])

        if show_detail:
            if discipline == "SP":
                base.extend([
                    row.get("courselap1", "-"),
                    row.get("courselap2", "-"),
                    row.get("courselap3", "-"),
                    row.get("range1", "-"),
                    row.get("range2", "-"),
                    row.get("shoot1", "-"),
                    row.get("shoot2", "-"),
                    row.get("prone1", "-"),
                    row.get("standing1", "-"),
                ])
            else:
                base.extend([
                    row.get("courselap1", "-"),
                    row.get("courselap2", "-"),
                    row.get("courselap3", "-"),
                ])
                if laps >= 4:
                    base.append(row.get("courselap4", "-"))
                if laps >= 5:
                    base.append(row.get("courselap5", "-"))
                base.extend([
                    row.get("range1", "-"),
                    row.get("range2", "-"),
                    row.get("range3", "-"),
                    row.get("range4", "-"),
                    row.get("shoot1", "-"),
                    row.get("shoot2", "-"),
                    row.get("shoot3", "-"),
                    row.get("shoot4", "-"),
                    row.get("prone1", "-"),
                    row.get("prone2", "-"),
                    row.get("standing1", "-"),
                    row.get("standing2", "-"),
                ])

        if show_sort_rank:
            base.insert(0, idx)
        render_rows.append(base)

    row_styles = None
    if is_pretty_output(args):
        if getattr(args, "highlight_wc", False):
            cat_id = (payload.get("Competition") or {}).get("catId", "").upper()
            if cat_id in ("SM", "SW"):
                wc_rank_map = _get_wc_rank_map(cat_id, 6)
                row_styles = [rank_style(wc_rank_map.get(row.get("ibu_id"))) for row in rows]
        if row_styles is None:
            row_styles = [rank_style(row.get("rank")) for row in rows]
    highlight_headers = None
    if show_sort_rank and sort_header and sort_header in headers:
        highlight_headers = [headers.index(sort_header)]
    render_table(
        headers,
        render_rows,
        pretty=is_pretty_output(args),
        row_styles=row_styles,
        highlight_headers=highlight_headers,
    )
    return 0
